{*************************************************************************}
{*   BZip2Module                                                         *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Data\Compress\BZip2\BZip2Module.pas                                 *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 17th February 2003  (17.02.2003)                           }
{*************************************************************************}


{This is implementation of BZip2 compression/decompression module.
approx tests (compared to ZLibEx): 30% better compression,
2,5 times slower compess, 4 times slower decompress. }


unit BZip2Module;


interface


uses
  Sys, Objects, Storage, BZip2;


type


  {Compression module using BZIP2. Can compress/decomress data.
  BZIP is not very fast but has excelent compress ratio.}
  TBZip2Module = class(TCompressModule)
  public

    constructor Create; override;
    destructor Destroy; override;

    {Compresses InData buffer and returns compressed data in OutData and
     size in OutSize. Returns true if action was succesful.}
    function CompressBuffer(InData: Pointer; InSize: Long; var OutData: Pointer;
      var OutSize: Long): boolean; override;

    {Decompresses InData buffer and returns decompressed data in OutData and
    size in OutSize. Returns true if action was succesful.}
    function DecompressBuffer(InData: Pointer; InSize: Long; var OutData:
      Pointer; var OutSize: Long): boolean; override;

  published

  end;


const

  sBZip2Name = 'BZip2 Module';
  sBZip2Desc = 'Provides BZip2 data compression/decompression';
  sBZip2Code:TIDCode = 'CMP-BZ2';


var

  BZip2ModuleInfo:TModuleInfo;


implementation


{*************************************************************************}
{                        TZLibModule Methods                              }
{*************************************************************************}


constructor TBZip2Module.Create;
begin
  inherited Create;
  FCanCompress := true;
  FCanDecompress := true;

  FInfo := BZip2ModuleInfo;
end;

destructor TBZip2Module.Destroy;
begin

  inherited Destroy;
end;

function TBZip2Module.CompressBuffer(InData: Pointer; InSize: Long;
  var OutData: Pointer; var OutSize: Long): boolean;
begin
  Result:=true;
  try
    BZip2.CompressBuffer(InData, InSize, OutData, OutSize);
  except
    RaiseError(ErrString(Self, 'CompressBuffer',
      'Error when compressing data! Data can be corrupted.'),ECompress);
    Result:=false;
  end;
end;

function TBZip2Module.DecompressBuffer(InData: Pointer; InSize: Long;
  var OutData: Pointer; var OutSize: Long): boolean;
begin
  Result:=true;
  try
    BZip2.DecompressBuffer(InData, InSize, OutData, OutSize);
  except
    RaiseError(ErrString(Self, 'DecompressBuffer',
      'Error when decompressing data! Data can be corrupted or uncompressed.'),ECompress);
    Result:=false;
  end;
end;


initialization

  BZip2ModuleInfo:=GetDefaultIntModuleInfo;
  BZip2ModuleInfo.Name:=sBZip2Name;
  BZip2ModuleInfo.Description:=sBZip2Desc;
  BZip2ModuleInfo.IDCode:=sBZip2Code;
  RegisterIntModule(@BZip2ModuleInfo,TBZip2Module);

end.

