{*************************************************************************}
{*   ZLibModule                                                          *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Data\Compress\ZLib\ZLibModule.pas                                   *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 17th February 2003  (17.02.2003)                           }
{*************************************************************************}


{This is implementation of Zlib compression/decompression module.}


unit ZLibModule;


interface


uses
  Sys, Objects, Storage, SysUtils,
  {Using ZLibEx unit - compression is about 5x - 15x faster than
  standard ZLib.pas shipped with Delphi}
  ZLibEx;


type


  {Compression module using ZLIB. Can compress/decomress data.
  ZLIB has very good speed (ZLibEx unit - many optimizations) and good compress ratio.}
  TZLibModule = class(TCompressModule)
  public

    constructor Create; override;
    destructor Destroy; override;

    {Compresses InData buffer and returns compressed data in OutData and
     size in OutSize. Returns true if action was succesful.}
    function CompressBuffer(InData: Pointer; InSize: Long; var OutData: Pointer;
      var OutSize: Long): boolean; override;

    {Decompresses InData buffer and returns decompressed data in OutData and
    size in OutSize. Returns true if action was succesful.}
    function DecompressBuffer(InData: Pointer; InSize: Long; var OutData:
      Pointer; var OutSize: Long): boolean; override;

  published

  end;


const

  sZLibName = 'ZLib Module';
  sZLibDesc = 'Provides ZLib data compression/decompression';
  sZLibCode:TIDCode = 'CMP-ZLIB';


var

  ZLibModuleInfo:TModuleInfo;


implementation


{*************************************************************************}
{                        TZLibModule Methods                              }
{*************************************************************************}


constructor TZLibModule.Create;
begin
  inherited Create;
  FCanCompress := true;
  FCanDecompress := true;

  FInfo := ZLibModuleInfo;
end;

destructor TZLibModule.Destroy;
begin

  inherited Destroy;
end;

function TZLibModule.CompressBuffer(InData: Pointer; InSize: Long;
  var OutData: Pointer; var OutSize: Long): boolean;
begin
  Result:=true;
  try
    ZLibEx.CompressBuffer(InData, InSize, OutData, OutSize);
  except
    RaiseError(ErrString(Self, 'CompressBuffer',
      'Error when compressing data! Data can be corrupted.'),ECompress);
    Result:=false;
  end;
end;

function TZLibModule.DecompressBuffer(InData: Pointer; InSize: Long;
  var OutData: Pointer; var OutSize: Long): boolean;
begin
  Result:=true;
  try
    ZLibEx.DecompressBuffer(InData, InSize, OutData, OutSize);
  except
    RaiseError(ErrString(Self, 'DecompressBuffer',
      'Error when decompressing data! Data can be corrupted or uncompressed.'),ECompress);
    Result:=false;
  end;
end;

initialization

  ZLibModuleInfo:=GetDefaultIntModuleInfo;
  ZLibModuleInfo.Name:=sZLibName;
  ZLibModuleInfo.Description:=sZLibDesc;
  ZLibModuleInfo.IDCode:=sZLibCode;
  RegisterIntModule(@ZLibModuleInfo,TZLibModule);

end.

