{*************************************************************************}
{*   SysObjects                                                          *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Sys\SysObjects.pas                                                  *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 16th February 2003  (16.02.2003)                           }
{*************************************************************************}


{Contains various classes, objects, interfaces (Modules, Engines, Lists, ...)
and class manipulation functions. Also
extends some std Delphi classes (So you don't need to add "Classes, Objects"
to uses but only "Objects"). Contains some lists from ContNrs.pas.
Unit is not called Classes.pas because of Delphi}


unit Objects;


{DONE -o Galfar : Add information from TExternalInfo into TModule class}
{DONE -o Galfar : Try templates from article}
{DONE -o Galfar : Make sure that external modules work}
{DONE -o Galfar : How to access module interfaces from exernal libs?}
{TODO -o Galfar : Create Simple interface for TList storing all objects living}
{DONE -o Galfar : Create own InterfaceList. Inspired by TCustomObjList and
descendants}
{TODO -o Galfar : Create something like TMessageHandler like in old engine.
This is for sending messages to world and all objects which are registered
to this message will be performed}
{TODO -o Galfar : Create ModEngine for only one active Module at time
renderers,players, ...}
{TODO -o Galfar : When loading modules from packages and they contains
same units error - use compiled with main package}
{TODO -o Galfar : Create TList override which will not cause exceptions
if index is invalid but will use RaiseError}
{DONE -o Galfar : create restricted stringlist for FSupportedCodes}
{TODO -o Galfar : Create TStringList override which will not cause exceptions
if index is invalid but will use RaiseError}

{TODO -o Galfar : it will be necessary to create own TList and TStringList.
Change names to avoid conflicts with VCL.}


interface


uses
  Sys, SysUtils, Classes;


type


{*************************************************************************}
{         Delphi/Kylix standard classes references/extensions             }
{*************************************************************************}


  {Misc classes}
  TPersistent = Classes.TPersistent;

  TThread = Classes.TThread;

  {Exception classes}
  Exception = SysUtils.Exception;

  EFCreateError = Classes.EFCreateError;
  EFOpenError = Classes.EFOpenError;
  EFilerError = Classes.EFilerError;
  EReadError = Classes.EReadError;
  EWriteError = Classes.EWriteError;
  EClassNotFound = Classes.EClassNotFound;
  EMethodNotFound = Classes.EMethodNotFound;
  EInvalidImage = Classes.EInvalidImage;
  EResNotFound = Classes.EResNotFound;
  EListError = Classes.EListError;
  EBitsError = Classes.EBitsError;
  EStringListError = Classes.EStringListError;
  EComponentError = Classes.EComponentError;
  EParserError = Classes.EParserError;
  EOutOfResources = Classes.EOutOfResources;
  EInvalidOperation = Classes.EInvalidOperation;



const

  {Stream Types/Consts}
  fmCreate = Classes.fmCreate;

  soFromBeginning = 0;
  soFromCurrent = 1;
  soFromEnd = 2;


type

  {Standard Streams}
  TStream = Classes.TStream;

  TCustomMemoryStream = Classes.TCustomMemoryStream;

  TMemoryStream = Classes.TMemoryStream;

  TResourceStream = Classes.TResourceStream;

  THandleStream = Classes.THandleStream;

  TStreamAdapter = Classes.TStreamAdapter;

  TFileStream = Classes.TFileStream;


  {Lists}
  TList = Classes.TList;

  TThreadList = Classes.TThreadList;

  TInterfaceList = Classes.TInterfaceList;

  IInterfaceList = Classes.IInterfaceList;

  TStrings = Classes.TStrings;


  {Borland's TClassList taken from ContNRS.pas.}
  TClassList = class(TList)
  protected
    function GetItems(Index: Long): TClass;
    procedure SetItems(Index: Long; AClass: TClass);
  public

    function Add(AClass: TClass): Long;
    function Extract(Item: TClass): TClass;
    function Remove(AClass: TClass): Long;
    function IndexOf(AClass: TClass): Long;
    function First: TClass;
    function Last: TClass;
    procedure Insert(Index: Long; AClass: TClass);

    property Items[Index: Long]: TClass read GetItems write SetItems; default;
  end;


  {Borland's TOrderedList taken from ContNRS.pas.
  Clear method was added.}
  TOrderedList = class(TObject)
  private
    FList: TList;
  protected
    procedure PushItem(AItem: Pointer); virtual; abstract;
    function PopItem: Pointer; virtual;
    function PeekItem: Pointer; virtual;
    property List: TList read FList;
  public
    constructor Create;
    destructor Destroy; override;

    procedure Clear; virtual;

    function Count: Integer;
    function AtLeast(ACount: Integer): Boolean;
    function Push(AItem: Pointer): Pointer;
    function Pop: Pointer;
    function Peek: Pointer;
  end;


  {Borland's TStack taken from ContNRS.pas.}
  TStack = class(TOrderedList)
  protected
    procedure PushItem(AItem: Pointer); override;
  end;


  {Borland's TObjectStack taken from ContNRS.pas.
  Object ownership was added.}
  TObjectStack = class(TStack)
  private
    FOwnsObjects: Boolean;
  public
    constructor Create; overload;
    constructor Create(AOwnsObjects: Boolean); overload;

    procedure Clear; override;

    function Push(AObject: TObject): TObject;
    function Pop: TObject;
    function Peek: TObject;

    {If set to true list frees all objects removed from list(when Stack is destroyed)}
    property OwnsObjects: Boolean read FOwnsObjects write FOwnsObjects;
  end;


  {Borland's TQueue taken from ContNRS.pas.}
  TQueue = class(TOrderedList)
  protected
    procedure PushItem(AItem: Pointer); override;
  end;


  {Borland's TObjectQueue taken from ContNRS.pas.
  Object ownership was added.}
  TObjectQueue = class(TQueue)
  private
    FOwnsObjects: Boolean;
  public
    constructor Create; overload; virtual;
    constructor Create(AOwnsObjects: Boolean); overload;

    procedure Clear; override;

    function Push(AObject: TObject): TObject;
    function Pop: TObject;
    function Peek: TObject;

    {If set to true list frees all objects removed from list(when Queue is destroyed)}
    property OwnsObjects: Boolean read FOwnsObjects write FOwnsObjects;
  end;


{*************************************************************************}
{                     Base Classes/Interfaces                             }
{*************************************************************************}


type

  EObjects = class(Exception);


  {Interface for TBaseObject.}
  IBase = interface(IAbstract)
    ['{14B88323-F8CB-458B-B3A6-1D7151F53BA8}']

    function GetTag: Long;
    procedure SetTag(Value: Long);

    property Tag: Long read GetTag write SetTag;

  end;


  {This class is base of all.}
  TBaseObject = class(TAbstractObject, IBase)
  protected

    FTag: Long;

    function GetTag: Long;

    procedure SetTag(Value: Long);

  public

    constructor Create; override;
    destructor Destroy; override;

  published

    {Put some custom number here.}
    property Tag: Long read GetTag write SetTag;

  end;


  {Reference to base classes}
  TBaseClass = class of TBaseObject;


  {Interface for TSingleton.}
  ISingleton = interface(IBase)
    ['{2C7686E8-5F35-4A97-BDC8-437968969541}']

    function GetDescription: string;
    function GetInitialized: boolean;
    procedure SetDescription(const Value: string);

    property Description: string read GetDescription write SetDescription;
    property Initialized: boolean read GetInitialized;

  end;


  {This class represents object with only one instance. Mainly
  used as engines/modules which are each unique.}
  TSingleton = class(TBaseObject, ISingleton)
  protected

    FDescription: string;
    FInitialized: boolean;

    function GetDescription: string; virtual;
    function GetInitialized: boolean; virtual;

    procedure SetDescription(const Value: string); virtual;

    {Use Initialize to set some singleton values or create some objects.
    It is called after all constructors.}
    procedure Initialize; virtual;

    {Use Finalize to set some values or free some objects.
    It is called before all destructors.}
    procedure Finalize; virtual;

  public

    constructor Create; override;
    destructor Destroy; override;

    {Initialize method is called here, after all constructors are executed}
    procedure AfterConstruction; override;

    {Finalize method is called here, before first destructor is executed}
    procedure BeforeDestruction; override;

  published

    {Desciption of Singleton's main purpose.}
    property Description: string read GetDescription write SetDescription;

    {Indicates if singleton is initialized}
    property Initialized: boolean read GetInitialized;

  end;


  {Reference to singleton classes}
  TSingletonClass = class of TSingleton;


{*************************************************************************}
{                    Module Types/Classes/Interfaces                      }
{*************************************************************************}


const

  {Names of functions which are required in external module libraries.}
  sModuleCount = 'ModuleCount';
  sModuleInfo = 'ModuleInfo';
  sModuleExtInfo = 'ModuleExtInfo';

  {Mask which is used when searching for modules in External Module Libraries}
  sModuleFileMask = 'mod*.*';


type

  EModule = class(EObjects);

  {This code is used for identification of modules e.g. 'IMG-DCIF','MUS-MIDI'.
  First three characters determine type of Module.
  It is case-sensitive.
  IMG is ImageModule, MUS is MusicModule, ...
  These chars are defined near declaration of adequate module.
  So IMG is in Images.pas}
  TIDCode = array[0..7] of Char;

  {(*}

  {This enum describes style of module implementation.}
  TModuleStyle = (
    msInternal = 0,  // Module is internal. It is compiled with entire engine and
                     // module's source is in subdir of dir containing source
                     // of module's abstract ancestor class. For example:
                     // TJpgModule is in JpgModule.pas which is in
                     // Engine\Data\Images\Jpg directory
    msLowLevel = 1,  // Module is wrapper to low-level functions in external library
    msAbsWrapper = 2,// Module is abstract unassigned wrapper to low-level module.
                     // No instance of this module is ever created, but when
                     // creating instance of wrapper to specific low-level module
                     // ClassRef from this module is used (based on IDCode
                     // agreement)
    msInterface = 3, // Module is accessed trough  interface from ext. library
    msClassInst = 4, // Module library returns created instance of module's class
    msClassRef = 5,  // Module library returns reference to module's class.
                     // Instance of this class must be created then
    msForce32 = MaxInt // to compile as 32bit integer (for compatibility)
    );

  {Record with information about module. Mainly for communicating with
  modules in External Module libraries.}
  TModuleInfo = packed record
    Name: array[0..31] of Char;          // module name e.g. "Zip Module".
                                         // Names are unique !!
                                         // Remember this when creating new one.
    Description: array[0..95] of Char;   // short module desc e.g.
                                         // "Provides access to Zip (*.zip)
                                         // archives"
    LibName: packed array[0..31] of Char;// name of module library.
                                         // Must begin with "Mod"
                                         // e.g. "ModZipArchive.dll"
    Creator: array[0..63] of Char;       // author name, group, ...
                                         // e.g. "Hasan Ahmed,
                                         // Taliban Explosive Computing"
    Language: array[0..15] of Char;      // language which this module
                                         // was written in e.g. "Delphi 7"
    IDCode: TIDCode;                     // see info about TIDCode
    Style: TModuleStyle;                 // indicates style of module
                                         // implementation
    Version: TVersionInfo;               // current module version
    Platform: TPlatform;                 // platform on which module is
                                         // able to run
    RequiresLib: boolean;                // if true, module requires
                                         // another library (BPL/DLL/SO) to run
  end;

  PModuleInfo = ^TModuleInfo;

  {*)}

  {Function types which are called from module libraries. They are declared as
  cdecl so they must be marked as cdecl in ext. libaries too.
  It is because of some compilers (mainly C++) exports functions
  under different name than declared (use __cdecl and extern "c" to
  avoid this).
  Each module library - even high level - must contain these three }

  {Returns total count of module implementors in library.}
  TModuleCountProc = procedure(var Count: Long); cdecl;

  {Returns base information of given module.}
  TModuleInfoProc = procedure(Index: Long; var Info: TModuleInfo); cdecl;

  {Returns extended information on given module.
  Differs for each Module descendant classes. Pointer
  to record with fields and functions for low-level functions or
  Pointer to function which creates instance of high-level module.
  Functions returned by ExtInfo (in structure or directly) all uses
  register calling (__fastcall), because this is Delphi's native
  and here we don't need exact names no more.
  !!! Don't call FreeMem on returned Pointer}
  TModuleExtInfoProc = procedure(Index: Long; var ExtInfo: Pointer); cdecl;


  {Interface for TModule.}
  IModule = interface(ISingleton)
    ['{05546BD4-D3A3-49C3-A2F0-FE5117E254A6}']

    function GetModuleInfo: TModuleInfo;

    property ModuleInfo: TModuleInfo read GetModuleInfo;
  end;

  PIModule = ^IModule;


  {This interface is used for declarations of wrapper-to-lowlevel modules.
  External low-level module wrappers must be registered
  same as internal modules!!!!!!
  Module register then according to IDCode (of low-level mod) selects appropiate
  IExternalModule implemenating class and creates its instance.
  Original Info about wrappers stays in ModuleRegister for future usage.
  See TExtCompressModule in Storage.pas for details.}
  IExternalModule = interface(IModule)
    ['{AAB36F6C-381D-42DC-B463-5898579C7D91}']

    {In this function all data and functions for given module are
    loaded from library defined by Handle. Index identifies module
    in multi-module libraries if GetModuleCount(Handle)>1.
    Returns true if Module succesfully loaded props/funcs from
    library.}
    function LoadFromLib(Handle: THandle; Index: Long = 0): boolean;

  end;


  {This class represents module (plugin) object. For usage see TEngine.
  Remember that each Module should be used independently.
  No need for Owner engine. However, it can use another module or
  engine which is lower level than Owner engine. Example:
  BZ2Archive module can use TTARArchive to open Tar archive
  contained in BZ2 archive (because BZ2 can pack only one file it is used
  for storing archives like Tar).
  Or TDungeonsAndDragons module should use FileIO engine to load rule files,
  Script engine to compile scripts,...}
  TModule = class(TSingleton, IModule)
  protected

    FInfo: TModuleInfo;

    function GetModuleInfo: TModuleInfo;
    function GetName: string; override;
    function GetDescription: string; override;
    function GetIDCode: TIDCode; virtual;

    procedure SetName(const Value: string); override;
    procedure SetDescription(const Value: string); override;
    procedure SetIDCode(const Value: TIDCode); virtual;

    {Default initializator sets Name and Desc to values
    which are set in constuctor to ModuleInfo.}
    procedure Initialize; override;

  public


    {In constructor ModuleInfo is set to default parameters. Change in
    descendants to actual values.}
    constructor Create; override;
    destructor Destroy; override;

    {Add here Module's functions.
    All should return True if they are successful and False
    if they are not - It is very important because TModEngines
    tries another module if one fails - and it fails if returning False.
    Put try .. except as a main body of every function in module.
    Some engines can rely on that all in-module
    exceptions are handled. They expect only True or False coming
    from Module. }
    // function DoSomething(params):boolean

    property ModuleInfo: TModuleInfo read GetModuleInfo;


  published

  end;


  {Reference to module classes}
  TModuleClass = class of TModule;


{*************************************************************************}
{                       Module Library functions                          }
{*************************************************************************}


{Returns true if library defined by Handle is valid Module Library.}
function IsValidModuleLib(Handle: THandle): boolean;

{Returns number of modules conatined in library defined by Handle}
function GetModuleCount(Handle: THandle): Long;

{Returns ModuleInfo of module conatined in library defined by Handle.
Index is number 0..GetModuleCount-1}
function GetModuleInfo(Handle: THandle; Index: Long): TModuleInfo;

{Returns extended ModuleInfo of module conatined in library defined by Handle.
Index is number 0..GetModuleCount-1.
For details look at TGetModuleExtInfo definition.}
function GetModuleExtInfo(Handle: THandle; Index: Long): Pointer;


{*************************************************************************}
{                         Engine Classes                                  }
{*************************************************************************}


type

  EEngine = class(EObjects);


  {Interface for TEngine.}
  IEngine = interface(ISingleton)
    ['{6AB1545F-4D51-4B48-8CB2-4397077026D7}']

  end;


  {This class represents engine object. Engines are almost everytime
  wrappers to one or more TModule objects. Each engine is specialized
  for different usage (Graphic, FileIO, Input, Scripting, ...
  This TEngine actually contains no module list - for engines
  that not need module support (like ModuleRegister). }
  TEngine = class(TSingleton, IEngine)
  protected

  public

    constructor Create; override;
    destructor Destroy; override;

  published

  end;


  {Reference to engine classes}
  TEngineClass = class of TEngine;


const

  {Max number of IDCode types supported by one
  TModEngine. It does not mean that TModEngine can contain
  only this number of IModules - it is number
  of ancestors of modules each defined by first
  three chars of ID code (e.g. CMP, IMG, ...).}
  MaxIDCodeTypes = 16;


type

  {(*}

  {Information about Modules' class, name, ... .
  Used in ModuleRegister for storing info about registered or imported modules.}
  TModuleRegisterInfo = record
    ModuleInfo:TModuleInfo;// See TModuleInfo declaration
                           // By value of Style following pointers are accessed.
    ClassRef: TModuleClass;// Reference to module's class (can be nil)
    Instance: TModule;     // Instance of module's class (can be nil)
    Intf: IModule;         // Module's Interface (can be nil)
    case Long of
    0: (Handle:THandle;    // Handle to external library (for externals)
       Index:Long);        // Index of module in external library
  end;

  PModuleRegisterInfo = ^TModuleRegisterInfo;

  {*)}

  TIModuleList = class;
  TRestrictedStrings = class;


  {Interface for TModEngine.}
  IModEngine = interface(IEngine)
    ['{CB7BE3A6-C38C-4D48-9072-EF9ED4F6F572}']

    function GetModules: TIModuleList;
    function GetModuleCount: Long;
    function GetModuleInfos: TList;
    function GetModuleInfo(Index: Long): TModuleRegisterInfo;
    function GetModuleInfoCount: Long;
    function GetAutoInitModules: boolean;
    function GetSupportedCodes: TRestrictedStrings;

    procedure SetAutoInitModules(Value: boolean);

    function FreeModule(Module: IModule): boolean;

    procedure ProcessModulesChange;
    function FindModuleByName(const Name: string): IModule;
    function FindModuleByCode(const IDCode: TIDCode): IModule;
    function FindModuleByClass(AClass: TModuleClass): IModule;

    property Modules: TIModuleList read GetModules;
    property ModuleCount: Long read GetModuleCount;
    property ModuleInfos: TList read GetModuleInfos;
    property ModuleInfo[Index: Long]: TModuleRegisterInfo read GetModuleInfo;
    property ModuleInfoCount: Long read GetModuleInfoCount;
    property AutoInitModules: boolean read GetAutoInitModules write
      SetAutoInitModules;
    property SupportedCodes: TRestrictedStrings read GetSupportedCodes;

  end;


  {Array of TList classes. See TModEngine.FRefLists for details.}
  TRefLists = array[0..MaxIDCodeTypes - 1] of TList;

  {Array of pointers to ActiveModules. See TModEngine.FRefActiveModules
  for usage.}
  TRefActiveModules = array[0..MaxIDCodeTypes - 1] of Pointer;


  {This class represents modular engine object. Engines are actually
  wrappers to one or more TModule objects. Each engine is specialized
  for different usage (Graphic, FileIO, Input, Scripting, ... }
  TModEngine = class(TEngine, IModEngine)
  protected

    FSupportedCodes: TRestrictedStrings;
    FModuleInfos: TList;
    FModules: TIModuleList;
    FAutoInitModules: boolean;

    {This variable can be used in engines which supports more than one
    ID types (like StorageEngine suports Compress, Coding, ... modules).
    In Modules list they are all created modules of any type and
    looking for e.g. 5th CompressModule every few miliseconds
    would be expensive. So you can store pointers to all
    CompressModules in FRefLists[0] and CodingModules in
    FRefLists[1].
    FRefLists are automatically created in constructor of engine and
    when new module is created/freed interface is
    added/removed to/from FRefLists.
    See TStorageEngine for usage of intfs store in here.
    !!Note:!! Interfaces are added to FRefLists[X] in CreateModule
    function. X is index of module's IDCode type in
    FSupportedCodes stringlist. So do not remove/exchange/...
    codes is this stringlist after some modules have been created.}
    FRefLists: TRefLists;

    {This array contains pointer to module which is
    now active (e.g. in StorageEngine it is module which
    will do compression when StorageEngine.CompressBuffer is called)
    It is array so every supported code can have active module (like
    FRefLists). However, in engines like ImageEngine it is ignored
    in functions like LoadImage() - it uses modulespecific
    for the format of image.
    See TStorageEngine for usage of FActiveModules.}
    FActiveModules: TRefActiveModules;

    function GetModules: TIModuleList;
    function GetModuleInfos: TList;
    function GetModuleCount: Long;
    function GetModuleInfo(Index: Long): TModuleRegisterInfo;
    function GetModuleInfoCount: Long;
    function GetAutoInitModules: boolean;
    function GetSupportedCodes: TRestrictedStrings;

    procedure SetAutoInitModules(Value: boolean);

    {ProcessModuleChange is called after creation here.}
    procedure Initialize; override;

    {Adds existing Module info from ModuleRegister to this engine and}
    procedure AddModule(Info: PModuleRegisterInfo); virtual;

    {This procedure creates new instance of Module and adds it to
    lists. This also updates ModuleRegister's info about this module
    and adds intf to FRefLists.}
    function CreateModule(Info: PModuleRegisterInfo): boolean; virtual;

    {This procedure frees instance of Module and removes it from
    lists. This also updates ModuleRegister's info about this module
    and removes it from FRefLists.}
    function FreeModule(Module: IModule): boolean; virtual;

    {Creates all modules registerd in ModuleInfos.}
    procedure CreateAllModules; virtual;

    {This procedure sets active module. It sets
    FRefActiveModules[X]:=Module where X is index of Module's
    IDCode type in FSupportedCodes.}
    procedure SetActiveModule(Module: IModule); virtual;

    {This returns index of IDCode type in FSupportedCodes.
    Result:=FSupportedCodes.IndexOf(Copy(IDCode,1,3));}
    function GetIDCodeTypeIndex(const IDCode: TIDCode): Long;


  public


    constructor Create; override;
    destructor Destroy; override;

    procedure Perform(var Message: TMsgRec); override;

    {This function is called when ModuleRegister adds/removes some
    modules. Supported modules are added via AddModule to this engine.}
    procedure ProcessModulesChange; virtual;

    {These functions return module found by given parameter.
    If such module is not found nil is returned.
    Searches trough created modules in Modules list.}
    function FindModuleByName(const Name: string): IModule; virtual;
    {In these two it returns first occurence of given Code or Class}
    function FindModuleByCode(const IDCode: TIDCode): IModule; virtual;
    function FindModuleByClass(AClass: TModuleClass): IModule; virtual;

    {Add here procedures which will access samely named procedures
    of created modules. Make sure that if one module fails another module
    with same ID code will try its luck. }
    //procedure CompressData()
    //procedure DecompressData()

    {List of all actually created modules.}
    property Modules: TIModuleList read GetModules;

    {Array of actually created modules with no typecasting needed.
    Add this array to descendants if you want to.}
    //property UserModules[Index:Long]:TUserModule read GetUserModule write SetUserModule;

    {Number of actually created modules}
    property ModuleCount: Long read GetModuleCount;

    {List of all registered modules of supported type.
    It is list of pointers to PModuleRegisterInfo records
    contained in ModuleRegister.}
    property ModuleInfos: TList read GetModuleInfos;

    {Returns TModuleRegisterInfo record from ModuleInfos list in
    more comfortable format.}
    property ModuleInfo[Index: Long]: TModuleRegisterInfo read GetModuleInfo;

    {Number of actually created modules}
    property ModuleInfoCount: Long read GetModuleInfoCount;

    {If this is True modules added to this engine are
    automatically created and initialized and added to ModuleList.
    Set this to false for engines where only active Module can exist
    (Renderers, Audio Players, ...}
    property AutoInitModules: boolean read GetAutoInitModules write
      SetAutoInitModules;

    {List of IDCode types (first three characters) which
    this engine supports.
    !!Note:!! Set all supported codes in constructor and
    then do not modify this list any more - See TModEngine.FRefLists
    for details.}
    property SupportedCodes: TRestrictedStrings read GetSupportedCodes;


  published

  end;


{*************************************************************************}
{                             Some new lists                              }
{*************************************************************************}


  {This is std Borland's stringlist with much faster comparsions.}
  TStringList = class(Classes.TStringList)
  protected
    function CompareStrings(const S1, S2: string): Long; override;
  public
  end;


  {This is same as TStringList only Sorted property is set to true in creation
  to enable Find method without additional code writing =
  instead of "S:=TStringlist.Create;S.Sorted:=true" , but S:=TSortedStrings.Create
  It is mainly for lazy persons (like me)}
  TSortedStrings = class(TStringList)
  public
    constructor Create;
  end;


  {This class is primary designed for TModEngines.
  It supports only adding and reading strings to list.
  Because if some IDCodes from TModEngines.FSupportedCodes
  were removed TModEngine would not work properly.
  All remove/exchange/... procedures are removed from this list}
  TRestrictedStrings = class(TStringList)
  protected

   {Only procedure which can remove something. For internal use.}
    procedure TrueClear;

  public

    constructor Create;
    destructor Destroy; override;

    procedure Clear; override;
    procedure Delete(Index: Integer); override;
    procedure Exchange(Index1, Index2: Integer); override;
    procedure Insert(Index: Integer; const S: string); override;
    procedure InsertObject(Index: Integer; const S: string;
      AObject: TObject); override;
    procedure Sort; override;
    procedure CustomSort(Compare: TStringListSortCompare); override;

  end;

  {TCustomObjectList is base class for lists of various obj types
  where you do not want to typecast  Obj:=TNuclearWarHead(List[I]).Launch.
  Supports object freeing in remove/delete/clear operations.}
  TCustomObjectList = class(TList)
  private
    FOwnsObjects: Boolean;
  protected
    procedure Notify(Ptr: Pointer; Action: TListNotification); override;
  public
    constructor Create; overload; virtual;
    constructor Create(AOwnsObjects: Boolean); overload;

    {Searching based on classtype. Classtype changes in inherited methods!!}
    function FindInstanceOf(AClass: TClass; AExact: Boolean = True; AStartAt:
      Long = 0): Long; overload;

    {Searching based on classmane. Classname is always the same}
    function FindInstanceOf(const AClassName: string; AStartAt: Long = 0): Long;
      overload;

    {If set to true list frees all objects removed from listn (Delete/Remove/Clear methods)}
    property OwnsObjects: Boolean read FOwnsObjects write FOwnsObjects;
  end;


  {Borland's TObjectList taken from ContNRS.pas and split between this and
  TCustomObjectList.}
  TObjectList = class(TCustomObjectList)
  protected
    function GetItem(Index: Long): TObject;
    procedure SetItem(Index: Long; AObject: TObject);
  public

    function Add(AObject: TObject): Long;
    function Extract(Item: TObject): TObject;
    function Remove(AObject: TObject): Long;
    function IndexOf(AObject: TObject): Long;
    procedure Insert(Index: Long; AObject: TObject);
    function First: TObject;
    function Last: TObject;

    property Items[Index: Long]: TObject read GetItem write SetItem; default;
  end;


  {Std object list for accessing TBaseObjects items without typecasting.
  Only members where typecasting would be required are changed.}
  TBaseList = class(TCustomObjectList)
  protected

    function GetItem(Index: Long): TBaseObject;
    procedure SetItem(Index: Long; ABase: TBaseObject);

  public

    function First: TBaseObject;
    function Last: TBaseObject;

    property Items[Index: Long]: TBaseObject read GetItem write SetItem;
    default;

  end;


  {TCustomInterfaceList is base class for lists of various obj types
  where you do not want to typecast.
  Warning!! This list and his descendants don't increase
  Interface's RefCount in any way (in contrary with
  Classes.TInterfaceList which increases it)}
  TCustomInterfaceList = class(TList)
  public

    function IndexOf(const Item: IInterface): Integer; virtual;
    function Add(const Item: IInterface): Long; virtual;
    procedure Insert(Index: Integer; const Item: IInterface); virtual;
    function Remove(const Item: IInterface): Integer; virtual;

  end;


  {Std interface list for accessing IInterface items without typecasting.
  Only members where typecasting would be required are changed.}
  TIInterfaceList = class(TCustomInterfaceList)
  protected

    function GetItem(Index: Long): IInterface;
    procedure SetItem(Index: Long; Value: IInterface);

  public

    function First: IInterface;
    function Last: IInterface;

    property Items[Index: Long]: IInterface read GetItem write SetItem; default;

  end;


  {Std interface list for accessing IBase interface items without typecasting.
  Only members where typecasting would be required are changed.}
  TIBaseList = class(TCustomInterfaceList)
  protected

    function GetItem(Index: Long): IBase;
    procedure SetItem(Index: Long; Value: IBase);

  public

    function First: IBase;
    function Last: IBase;

    property Items[Index: Long]: IBase read GetItem write SetItem; default;

  end;


  {Std interface list for accessing IModule interface items without typecasting.
  Only members where typecasting would be required are changed.}
  TIModuleList = class(TCustomInterfaceList)
  protected

    function GetItem(Index: Long): IModule;
    procedure SetItem(Index: Long; Value: IModule);

  public

    function First: IModule;
    function Last: IModule;

    property Items[Index: Long]: IModule read GetItem write SetItem; default;

  end;


{*************************************************************************}
{                   Module Info Type/Functions/Objects                    }
{*************************************************************************}


  {Interface for TModuleRegister}
  IModuleRegister = interface(IEngine)
    ['{7F0441FC-DE0C-4458-BB95-66EAC160F98A}']

    function GetModuleInfos: TList;
    function GetModuleInfo(Index: Long): TModuleRegisterInfo;
    function GetModEngines: TBaseList;

    function RegisterModule(Info: PModuleRegisterInfo): boolean;
    function CreateModule(Info: PModuleRegisterInfo): IModule;
    procedure SearchExternals;
    procedure SearchDirectory(const Path: string);

    property ModuleInfos: TList read GetModuleInfos;
    property ModuleInfo[Index: Long]: TModuleRegisterInfo read GetModuleInfo;
    property ModEngines: TBaseList read GetModEngines;

  end;


  {This class stores and manages information about all accessible
  modules. Internal modules are registered in declaration and
  external modules are loaded on startup.
  When new module is added, ModuleRegister notifies all
  ModEngines so they can search trough Register for new
  supported modules. }
  TModuleRegister = class(TEngine, IModuleRegister)
  protected

    FModuleInfos: TList;
    FModEngines: TBaseList;

    function GetModuleInfos: TList;
    function GetModuleInfo(Index: Long): TModuleRegisterInfo;
    function GetModEngines: TBaseList;

    {This function goes over ModuleInfos until Info with Style
    msAbsWrapper and Code (first 3 chars of course) which agrees with
    Code in Info struct. Info is left unchanged. Found class ref is returned
    if found or nil if not found. }
    function FindWrapperClassRef(Info: PModuleRegisterInfo): TModuleClass;

  public

    constructor Create; override;
    destructor Destroy; override;

    procedure Perform(var Message: TMsgRec); override;

    {Adds info about new module and checks for duplicates
    (Can have same IDCode but not same Class (except external low-level)
    and Name). If duplicate exists, given info is not added.
    Then notifies all ModEngines. Returns true if given info was
    added to list otherwise returns false.}
    function RegisterModule(Info: PModuleRegisterInfo): boolean;

    {Returns RegisterModuleInfo for module in External Module Library.}
    function ExternalInfo(Handle: THandle; Index: Long):
      PModuleRegisterInfo;

    {Creates instance of new module based on Info.
    This Info is then changed according to created instance.
    Returns interafce to new module.
    (This function is often called form TModEngine.CreateModule) }
    function CreateModule(Info: PModuleRegisterInfo): IModule;

    {This function is automatically called after app's startup.
    It goes over default module directories and here looks
    for Module Libraries. Every module from every library is then
    registered.}
    procedure SearchExternals;

    {Recursive search trough directory for External Module Libs.
    Uses mask declared as sModulFileMask 'mod*.*' .}
    procedure SearchDirectory(const Path: string);


    {List of info structures for all known modules.}
    property ModuleInfos: TList read GetModuleInfos;

    {Returns TModuleRegisterInfo record from ModuleInfos list in
    more comfortable format.}
    property ModuleInfo[Index: Long]: TModuleRegisterInfo read GetModuleInfo;

    {List of all TModEngines to which notification is send
    when some module is added or removed.}
    property ModEngines: TBaseList read GetModEngines;


  published

  end;


{*************************************************************************}
{                         Other functions                                 }
{*************************************************************************}


{Allocates and fills new info struct. Result must be freed after usage by
FreeModuleRegisterInfo.}
function ModuleRegisterInfo(Info: PModuleInfo; ClassRef: TModuleClass; Instance:
  TModule; Intf: IModule):
  PModuleRegisterInfo;

{Frees memory used by Info and sets pointer to nil}
procedure FreeModuleRegisterInfo(var Info: PModuleRegisterInfo);

{Function for registration of internal modules.
Add it to initialization section of unit in which module is declared.}
procedure RegisterIntModule(Info: PModuleInfo; ClassRef: TModuleClass);

{Returns default info for internal modules.
Use it before calling RegisterIntModule and only change
fields specific for your internal module.}
function GetDefaultIntModuleInfo: TModuleInfo;

{Returns true if IDCodes are identical (case-sensitive)}
function SameIDCodes(const ID1, ID2: TIDCode): boolean;

{Returns true IDCode types are the same - first three
chars are identical.}
function SameIDType(const IDType, IDCode: TIDCode): boolean;

{Returns string representation of Module style}
function ModuleStyleToStr(Style:TModuleStyle):string;


const

  {(*}

  {Objects.pas messages(128 - 255)
  All messages defined in Sys directory must begin with SM prefix.}
  SM_ModuleRegisterChanged = 00128 + 0005; // module was added to or
                                           // removed from ModuleRegister

  {*)}


var

  {Interface to ModuleRegisterInstance}
  ModuleRegister: IModuleRegister;


implementation


uses
  Strings, Maths;


type

  TModuleStyleNames = array[msInternal .. msClassRef] of string;

var

  {Instance of TModuleRegister for internall use in this unit.}
  ModuleRegisterInstance: TModuleRegister;


  DefaultIntModuleInfo: TModuleInfo = (
    Name: 'Abstract Module';
    Description: 'Module with unknown capabilities';
    LibName: 'Main executable';
    Creator: 'Marek Mauder, Vampyre SoftWorks';
    Language: 'Delphi 7';
    IDCode: 'MOD-UNKN';
    Style: msInternal;
    Version: (Version: 0);
    Platform: pfIndependent;
    RequiresLib: false);


const

  {Names for style>string conversions}
  ModuleStyleNames:TModuleStyleNames= (
    'Internal',
    'Low Level',
    'Abstract Wrapper',
    'Interface',
    'Class Instance',
    'Class Reference');


{*************************************************************************}
{                         TClassList Methods                              }
{*************************************************************************}


function TClassList.Add(AClass: TClass): Long;
begin
  Result := inherited Add(AClass);
end;

function TClassList.Extract(Item: TClass): TClass;
begin
  Result := TClass(inherited Extract(Item));
end;

function TClassList.First: TClass;
begin
  Result := TClass(inherited First);
end;

function TClassList.GetItems(Index: Long): TClass;
begin
  Result := TClass(inherited Items[Index]);
end;

function TClassList.IndexOf(AClass: TClass): Long;
begin
  Result := inherited IndexOf(AClass);
end;

procedure TClassList.Insert(Index: Long; AClass: TClass);
begin
  inherited Insert(Index, AClass);
end;

function TClassList.Last: TClass;
begin
  Result := TClass(inherited Last);
end;

function TClassList.Remove(AClass: TClass): Long;
begin
  Result := inherited Remove(AClass);
end;

procedure TClassList.SetItems(Index: Long; AClass: TClass);
begin
  inherited Items[Index] := AClass;
end;


{*************************************************************************}
{                         TOrderedList Methods                            }
{*************************************************************************}


constructor TOrderedList.Create;
begin
  inherited Create;
  FList := TList.Create;
end;

destructor TOrderedList.Destroy;
begin
  Clear;
  List.Free;
  inherited Destroy;
end;

procedure TOrderedList.Clear;
begin
  while List.Count > 0 do
    List.Delete(0);
end;

function TOrderedList.AtLeast(ACount: integer): boolean;
begin
  Result := List.Count >= ACount;
end;

function TOrderedList.Peek: Pointer;
begin
  Result := PeekItem;
end;

function TOrderedList.Pop: Pointer;
begin
  Result := PopItem;
end;

function TOrderedList.Push(AItem: Pointer): Pointer;
begin
  PushItem(AItem);
  Result := AItem;
end;

function TOrderedList.Count: Integer;
begin
  Result := List.Count;
end;

function TOrderedList.PeekItem: Pointer;
begin
  Result := List[List.Count - 1];
end;

function TOrderedList.PopItem: Pointer;
begin
  Result := PeekItem;
  List.Delete(List.Count - 1);
end;


{*************************************************************************}
{                         TStack Methods                                  }
{*************************************************************************}


procedure TStack.PushItem(AItem: Pointer);
begin
  List.Add(AItem);
end;


{*************************************************************************}
{                         TObjectStack Methods                            }
{*************************************************************************}


constructor TObjectStack.Create;
begin
  inherited Create;
  FOwnsObjects := True;
end;

constructor TObjectStack.Create(AOwnsObjects: Boolean);
begin
  inherited Create;
  FOwnsObjects := AOwnsObjects;
end;

procedure TObjectStack.Clear;
var
  O: TObject;
begin
  while List.Count > 0 do
  begin
    O := List[0];
    List.Delete(0);
    O.Free;
  end;
end;

function TObjectStack.Peek: TObject;
begin
  Result := TObject(inherited Peek);
end;

function TObjectStack.Pop: TObject;
begin
  Result := TObject(inherited Pop);
end;

function TObjectStack.Push(AObject: TObject): TObject;
begin
  Result := TObject(inherited Push(AObject));
end;


{*************************************************************************}
{                         TQueue Methods                                  }
{*************************************************************************}


procedure TQueue.PushItem(AItem: Pointer);
begin
  List.Insert(0, AItem);
end;


{*************************************************************************}
{                         TObjectQueue Methods                            }
{*************************************************************************}


constructor TObjectQueue.Create;
begin
  inherited Create;
  FOwnsObjects := True;
end;

constructor TObjectQueue.Create(AOwnsObjects: Boolean);
begin
  inherited Create;
  FOwnsObjects := AOwnsObjects;
end;

procedure TObjectQueue.Clear;
var
  O: TObject;
begin
  while List.Count > 0 do
  begin
    O := List[0];
    List.Delete(0);
    O.Free;
  end;
end;

function TObjectQueue.Peek: TObject;
begin
  Result := TObject(inherited Peek);
end;

function TObjectQueue.Pop: TObject;
begin
  Result := TObject(inherited Pop);
end;

function TObjectQueue.Push(AObject: TObject): TObject;
begin
  Result := TObject(inherited Push(AObject));
end;


{*************************************************************************}
{                         TBaseObject Methods                             }
{*************************************************************************}


constructor TBaseObject.Create;
begin
  inherited Create;
  FTag := 0;

end;

destructor TBaseObject.Destroy;
begin

  inherited Destroy;
end;

function TBaseObject.GetTag: Long;
begin
  Result := FTag;
end;

procedure TBaseObject.SetTag(Value: Long);
begin
  FTag := Value;
end;


{*************************************************************************}
{                         TSingleton Methods                              }
{*************************************************************************}


constructor TSingleton.Create;
begin
  inherited Create;
  FDescription := 'Singleton with unknown capabilities';
  FInitialized := false;
end;

destructor TSingleton.Destroy;
begin

  inherited Destroy;
end;

procedure TSingleton.AfterConstruction;
begin
  // This is very important line in every AfterConstruction override
  // You see empty procedure in System.pas' TObject.AfterConstruction
  // but this is not exactly true. Some autochecking is done here ->
  // If inherited method is not called then RefCount after creation
  // will be 2 instead of 1.
  inherited AfterConstruction;
  Initialize;
end;

procedure TSingleton.BeforeDestruction;
begin
  Finalize;
  inherited BeforeDestruction;
end;

function TSingleton.GetDescription: string;
begin
  Result := FDescription;
end;

function TSingleton.GetInitialized: boolean;
begin
  Result := FInitialized;
end;

procedure TSingleton.SetDescription(const Value: string);
begin
  FDescription := Value;
end;

procedure TSingleton.Initialize;
begin
  FInitialized := true;
end;

procedure TSingleton.Finalize;
begin
  FInitialized := false;
end;


{*************************************************************************}
{                         TModule Methods                                 }
{*************************************************************************}


constructor TModule.Create;
begin
  inherited Create;
  FInfo := GetDefaultIntModuleInfo;
end;

destructor TModule.Destroy;
begin

  inherited Destroy;
end;

function TModule.GetModuleInfo: TModuleInfo;
begin
  Result := FInfo;
end;

function TModule.GetName: string;
begin
  Result := FInfo.Name;
end;

function TModule.GetDescription: string;
begin
  Result := FInfo.Description;
end;

function TModule.GetIDCode: TIDCode;
begin
  Result := FInfo.IDCode;
end;

procedure TModule.SetName(const Value: string);
begin
  StrPCopy(FInfo.Name, Value);
end;

procedure TModule.SetDescription(const Value: string);
begin
  StrPCopy(FInfo.Description, Value);
end;

procedure TModule.SetIDCode(const Value: TIDCode);
begin
  FInfo.IDCode := Value;
end;

procedure TModule.Initialize;
begin
  FName := FInfo.Name;
  FDescription := FInfo.Description;
end;


{*************************************************************************}
{                       Module Library functions                          }
{*************************************************************************}


function IsValidModuleLib(Handle: THandle): boolean;
begin
  Result := (GetProcAddress(Handle, sModuleCount) <> nil) and
    (GetProcAddress(Handle, sModuleInfo) <> nil) and
    (GetProcAddress(Handle, sModuleExtInfo) <> nil);
end;

function GetModuleCount(Handle: THandle): Long;
var
  CountProc: TModuleCountProc;

begin
  Result := 0;
  if not IsValidModuleLib(Handle) then
    Exit;
  @CountProc := GetProcAddress(Handle, sModuleCount);
  if @CountProc <> nil then
    CountProc(Result);
end;

function GetModuleInfo(Handle: THandle; Index: Long): TModuleInfo;
var
  InfoProc: TModuleInfoProc;

begin
  FillChar(Result, SizeOf(TModuleInfo), 0);
  if not IsValidModuleLib(Handle) then
    Exit;
  @InfoProc := GetProcAddress(Handle, sModuleInfo);
  if @InfoProc <> nil then
    InfoProc(Index, Result);
end;

function GetModuleExtInfo(Handle: THandle; Index: Long): Pointer;
var
  InfoProc: TModuleExtInfoProc;

begin
  Result := nil;
  if not IsValidModuleLib(Handle) then
    Exit;
  @InfoProc := GetProcAddress(Handle, sModuleExtInfo);
  if @InfoProc <> nil then
    InfoProc(Index, Result);
end;


{*************************************************************************}
{                         TEngine Methods                                 }
{*************************************************************************}


constructor TEngine.Create;
begin
  inherited Create;
  FDescription := 'Engine with unknown capabilities';

end;

destructor TEngine.Destroy;
begin

  inherited Destroy;
end;


{*************************************************************************}
{                         TModEngine Methods                              }
{*************************************************************************}


constructor TModEngine.Create;
var
  I: Long;
begin
  inherited Create;
  ModuleRegister.ModEngines.Add(Self);

  FModules := TIModuleList.Create;
  FSupportedCodes := TRestrictedStrings.Create;
  FModuleInfos := TList.Create;

  for I := Low(FRefLists) to High(FRefLists) do
    FRefLists[I] := TList.Create;

  FAutoInitModules := true;
end;

destructor TModEngine.Destroy;
var
  I: Long;
begin
  for I := Low(FRefLists) to High(FRefLists) do
    FRefLists[I].Free;

  FModules.Free;
  FSupportedCodes.Free;
  FModuleInfos.Free;

  ModuleRegister.ModEngines.Extract(Self);
  inherited Destroy;
end;

procedure TModEngine.Initialize;
begin
  inherited Initialize;
  ProcessModulesChange;
end;

function TModEngine.GetModules: TIModuleList;
begin
  Result := FModules;
end;

function TModEngine.GetModuleInfos: TList;
begin
  Result := FModuleInfos;
end;

function TModEngine.GetModuleCount: Long;
begin
  Result := FModules.Count;
end;

function TModEngine.GetModuleInfo(Index: Long): TModuleRegisterInfo;
begin
  Result := PModuleRegisterInfo(FModuleInfos[Index])^;
end;

function TModEngine.GetModuleInfoCount: Long;
begin
  Result := FModuleInfos.Count;
end;

function TModEngine.GetAutoInitModules: boolean;
begin
  Result := FAutoInitModules;
end;

function TModEngine.GetSupportedCodes: TRestrictedStrings;
begin
  Result := FSupportedCodes;
end;

procedure TModEngine.SetAutoInitModules(Value: boolean);
begin
  FAutoInitModules := Value;
end;

procedure TModEngine.AddModule(Info: PModuleRegisterInfo);
var
  I: Long;
begin
  if Info = nil then
    Exit;

  // check for duplicates
  for I := 0 to FModuleInfos.Count - 1 do
    if PModuleRegisterInfo(FModuleInfos[I])^.ModuleInfo.Name =
      Info^.ModuleInfo.Name then
      Exit;

  FModuleInfos.Add(Info);
end;

function TModEngine.CreateModule(Info: PModuleRegisterInfo): boolean;
var
  Intf: IModule;
  I: Long;
begin
  Result := false;
  if Info = nil then
    Exit;

  // do not add existing module
  for I := 0 to FModules.Count - 1 do
    // here must be exct comparsion
    if FModules[I].ModuleInfo.Name = Info^.ModuleInfo.Name then
      Exit;

  try
    // calls ModuleRegister's CreateModule method
    Intf := ModuleRegister.CreateModule(Info);
    if Intf <> nil then
      // be sure that Intf is unique. Sometimes when
      // you add interface to object that is already
      // in list weird errors occur
    begin
      Modules.Add(Intf);

       // add intf to FRefLists
      I := GetIDCodeTypeIndex(Info^.ModuleInfo.IDCode);
      if I >= 0 then
        FRefLists[I].Add(Pointer(Intf));

       // if none module is active (after startup mainly) first
       // created module is set as active
      if FActiveModules[I] = nil then
        FActiveModules[I] := Pointer(Intf);

      Result := true;
    end;
  except
    RaiseError(ErrString(Self, 'CreateModule',
      Format('Error when creating new module (Name: %s, Class: %s).',
      [Info^.ModuleInfo.Name, Iff(Info^.ClassRef <> nil,
        Info^.ClassRef.ClassName,
        '<none>')])), EEngine);
  end;
end;

function TModEngine.FreeModule(Module: IModule): boolean;
var
  I, Idx, IRef: Long;
begin
  Result := false;
  if Module = nil then
    Exit;

  Idx := -1;
  for I := 0 to FModuleInfos.Count - 1 do
    if PModuleRegisterInfo(FModuleInfos[I])^.Intf = Module then
    begin
      Idx := I;
      Break;
    end;

  if Idx = -1 then
    Exit;

  FModules.Remove(Module);
  I := GetIDCodeTypeIndex(Module.ModuleInfo.IDCode);
  if I >= 0 then
  begin
    if FActiveModules[I] = Pointer(Module) then
    begin
      IRef := FRefLists[I].IndexOf(Pointer(Module));
      if IRef = -1 then
        FActiveModules[I] := nil
      else
        if IRef = 0 then
        begin
          if FRefLists[I].Count > 1 then
            FActiveModules[I] := FRefLists[I][IRef + 1]
          else
            FActiveModules[I] := nil
        end
        else
          FActiveModules[I] := FRefLists[I][IRef - 1]
    end;
    FRefLists[I].Remove(Pointer(Module));
  end;

  PModuleRegisterInfo(FModuleInfos[Idx])^.Intf := nil;
  // Instance is really freed when RefCount is 0
  PModuleRegisterInfo(FModuleInfos[Idx])^.Instance := nil;
end;

procedure TModEngine.CreateAllModules;
var
  I: Long;
begin
  for I := 0 to FModuleInfos.Count - 1 do
    CreateModule(FModuleInfos[I]);
end;

procedure TModEngine.SetActiveModule(Module: IModule);
var
  I: Long;
begin
  if Module = nil then
    Exit;
  I := GetIDCodeTypeIndex(Module.ModuleInfo.IDCode);
  if I >= 0 then
    FActiveModules[I] := Pointer(Module);
end;


function TModEngine.GetIDCodeTypeIndex(const IDCode: TIDCode): Long;
begin
  Result := FSupportedCodes.IndexOf(Copy(IDCode, 1, 3));
end;

procedure TModEngine.Perform(var Message: TMsgRec);
begin
  case Message.MsgID of
    SM_ModuleRegisterChanged: ProcessModulesChange;

  else
    inherited Perform(Message);
  end;
end;

procedure TModEngine.ProcessModulesChange;
var
  I: Long;
  Code: string;
begin
  // put here code which will go trough
  // ModuleRegister.ModulesInfo and pick up modules
  // of supported Class and ID.
  // e.g. CompressEngine will pick up all
  // modules which has 'CMP' IDCode
  for I := 0 to ModuleRegister.ModuleInfos.Count - 1 do
  begin
    Code :=
      Copy(PModuleRegisterInfo(ModuleRegister.ModuleInfos[I])^.ModuleInfo.IDCode,
      1, 3);
    if StrInList(Code, FSupportedCodes) then
      AddModule(PModuleRegisterInfo(ModuleRegister.ModuleInfos[I]))
  end;
  // if modules can be created right after registration then
  // create them.
  // Also there is need to process all low-level modules - they could not
  // be created before because their IExternalModule wrapper
  // should have not been registered when low-level module was about to be
  // created.
  if FAutoInitModules then
    CreateAllModules;
end;

function TModEngine.FindModuleByName(const Name: string): IModule;
var
  I: Long;
begin
  Result := nil;
  for I := 0 to FModules.Count - 1 do
    if Strings.SameStr(Name, Modules[I].ModuleInfo.Name) then
    begin
      Result := Modules[I];
      Exit;
    end;
end;

function TModEngine.FindModuleByCode(const IDCode: TIDCode): IModule;
var
  I: Long;
begin
  Result := nil;
  for I := 0 to FModules.Count - 1 do
    if IDCode = Modules[I].ModuleInfo.IDCode then
    begin
      Result := Modules[I];
      Exit;
    end;
end;

function TModEngine.FindModuleByClass(AClass: TModuleClass): IModule;
var
  I: Long;
begin
  Result := nil;
  for I := 0 to FModules.Count - 1 do
    if AClass = TModuleClass(GetIntfImplementor(Modules[I]).ClassType) then
    begin
      Result := Modules[I];
      Exit;
    end;
end;


{*************************************************************************}
{                         TStringList Methods                             }
{*************************************************************************}


function TStringList.CompareStrings(const S1, S2: string): Long;
begin
  if CaseSensitive then
    Result := CompStrCase(S1, S2)
  else
    Result := CompStr(S1, S2);
end;


{*************************************************************************}
{                         TSortedStrings Methods                             }
{*************************************************************************}


constructor TSortedStrings.Create;
begin
  inherited Create;
  Sorted := true;
end;


{*************************************************************************}
{                      TRestrictedStrings Methods                         }
{*************************************************************************}


constructor TRestrictedStrings.Create;
begin
  inherited Create;
  Duplicates := dupIgnore;

end;

destructor TRestrictedStrings.Destroy;
begin
  TrueClear;
  inherited Destroy;
end;

procedure TRestrictedStrings.Clear;
begin
end;

procedure TRestrictedStrings.Delete(Index: Integer);
begin
end;

procedure TRestrictedStrings.Exchange(Index1, Index2: Integer);
begin
end;

procedure TRestrictedStrings.Insert(Index: Integer; const S: string);
begin
end;

procedure TRestrictedStrings.InsertObject(Index: Integer; const S: string;
  AObject: TObject);
begin
end;

procedure TRestrictedStrings.Sort;
begin
end;

procedure TRestrictedStrings.CustomSort(Compare: TStringListSortCompare);
begin
end;

procedure TRestrictedStrings.TrueClear;
begin
  inherited Clear;
end;


{*************************************************************************}
{                         TCustomObjectList Methods                       }
{*************************************************************************}


constructor TCustomObjectList.Create;
begin
  inherited Create;
  FOwnsObjects := True;
end;

constructor TCustomObjectList.Create(AOwnsObjects: Boolean);
begin
  inherited Create;
  FOwnsObjects := AOwnsObjects;
end;

procedure TCustomObjectList.Notify(Ptr: Pointer; Action: TListNotification);
begin
  if OwnsObjects then
    if Action = lnDeleted then
      TObject(Ptr).Free;
  inherited Notify(Ptr, Action);
end;

function TCustomObjectList.FindInstanceOf(AClass: TClass; AExact: Boolean;
  AStartAt: Long): Long;
var
  I: Long;
begin
  Result := -1;
  for I := AStartAt to Count - 1 do
    if Items[I] = nil then
      Continue
    else
      if (AExact and
        (TObject(Items[I]).ClassType = AClass)) or
        (not AExact and
        TObject(Items[I]).InheritsFrom(AClass)) then
      begin
        Result := I;
        Break;
      end;
end;

function TCustomObjectList.FindInstanceOf(const AClassName: string; AStartAt:
  Long =
  0): Long;
var
  I: Long;
begin
  Result := -1;
  for I := AStartAt to Count - 1 do
    if Items[I] = nil then
      Continue
    else
      if TObject(Items[I]).ClassNameIs(AClassName) then
      begin
        Result := I;
        Break;
      end;
end;


{*************************************************************************}
{                         TObjectList Methods                             }
{*************************************************************************}


function TObjectList.Add(AObject: TObject): Long;
begin
  Result := inherited Add(AObject);
end;

function TObjectList.Extract(Item: TObject): TObject;
begin
  Result := TObject(inherited Extract(Item));
end;

function TObjectList.First: TObject;
begin
  Result := TObject(Get(0));
end;

function TObjectList.GetItem(Index: Long): TObject;
begin
  Result := Get(Index);
end;

function TObjectList.IndexOf(AObject: TObject): Long;
begin
  Result := inherited IndexOf(AObject);
end;

procedure TObjectList.Insert(Index: Long; AObject: TObject);
begin
  inherited Insert(Index, AObject);
end;

function TObjectList.Last: TObject;
begin
  Result := TObject(Get(Count - 1));
end;

function TObjectList.Remove(AObject: TObject): Long;
begin
  Result := inherited Remove(AObject);
end;

procedure TObjectList.SetItem(Index: Long; AObject: TObject);
begin
  Put(Index, AObject);
end;


{*************************************************************************}
{                         TBaseList Methods                               }
{*************************************************************************}


function TBaseList.First: TBaseObject;
begin
  Result := TBaseObject(Get(0));
end;

function TBaseList.GetItem(Index: Long): TBaseObject;
begin
  Result := Get(Index);
end;

function TBaseList.Last: TBaseObject;
begin
  Result := TBaseObject(Get(Count - 1));
end;

procedure TBaseList.SetItem(Index: Long; ABase: TBaseObject);
begin
  Put(Index, ABase);
end;


{*************************************************************************}
{                   TCustomInterfaceList Methods                          }
{*************************************************************************}


function TCustomInterfaceList.IndexOf(const Item: IInterface): Long;
begin
  Result := inherited IndexOf(Pointer(Item));
end;

function TCustomInterfaceList.Add(const Item: IInterface): Long;
begin
  Result := inherited Add(Pointer(Item));
end;

procedure TCustomInterfaceList.Insert(Index: Long; const Item: IInterface);
begin
  inherited Insert(Index, Pointer(Item));
end;

function TCustomInterfaceList.Remove(const Item: IInterface): Long;
begin
  Result := inherited Remove(Pointer(Item));
end;


{*************************************************************************}
{                       TInterfaceList Methods                            }
{*************************************************************************}


function TIInterfaceList.GetItem(Index: Long): IInterface;
begin
  Result := IInterface(Get(Index));
end;

procedure TIInterfaceList.SetItem(Index: Long; Value: IInterface);
begin
  Put(Index, Pointer(Value));
end;

function TIInterfaceList.First: IInterface;
begin
  Result := IInterface(Get(0));
end;

function TIInterfaceList.Last: IInterface;
begin
  Result := IInterface(Get(Count - 1));
end;


{*************************************************************************}
{                         TIBaseList Methods                              }
{*************************************************************************}


function TIBaseList.First: IBase;
begin
  Result := IBase(Get(0));
end;

function TIBaseList.GetItem(Index: Long): IBase;
begin
  Result := IBase(Get(Index));
end;

function TIBaseList.Last: IBase;
begin
  Result := IBase(Get(Count - 1));
end;

procedure TIBaseList.SetItem(Index: Long; Value: IBase);
begin
  Put(Index, Pointer(Value));
end;


{*************************************************************************}
{                         TIModuleList Methods                              }
{*************************************************************************}


function TIModuleList.First: IModule;
begin
  Result := IModule(Get(0));
end;

function TIModuleList.GetItem(Index: Long): IModule;
begin
  Result := IModule(Get(Index));
end;

function TIModuleList.Last: IModule;
begin
  Result := IModule(Get(Count - 1));
end;

procedure TIModuleList.SetItem(Index: Long; Value: IModule);
begin
  Put(Index, Pointer(Value));
end;


{*************************************************************************}
{                      TModuleRegister Methods                            }
{*************************************************************************}


constructor TModuleRegister.Create;
begin
  inherited Create;
  FModuleInfos := TList.Create;
  FModEngines := TBaseList.Create(false);
end;

destructor TModuleRegister.Destroy;
begin
  FModuleInfos.Free;
  FModEngines.Free;
  inherited Destroy;
end;

function TModuleRegister.GetModuleInfos: TList;
begin
  Result := FModuleInfos;
end;

function TModuleRegister.GetModuleInfo(Index: Long): TModuleRegisterInfo;
begin
  Result := PModuleRegisterInfo(FModuleInfos[Index])^;
end;

function TModuleRegister.GetModEngines: TBaseList;
begin
  Result := FModEngines;
end;

function TModuleRegister.FindWrapperClassRef(Info: PModuleRegisterInfo):
  TModuleClass;
var
  I: Long;
begin
  Result := nil;
  // looks for module with style msAbsWrapper which Code agrees with
  // Code of module described by Info
  for I := 0 to ModuleInfos.Count - 1 do
    if PModuleRegisterInfo(ModuleInfos[I])^.ModuleInfo.Style = msAbsWrapper then
      if SameIDType(PModuleRegisterInfo(ModuleInfos[I])^.ModuleInfo.IDCode,
        Info^.ModuleInfo.IDCode) then
      begin
        Result := PModuleRegisterInfo(ModuleInfos[I])^.ClassRef;
        Exit;
      end;
end;

procedure TModuleRegister.Perform(var Message: TMsgRec);
begin
  case Message.MsgID of
    SM_FreeSelf: ; // ModuleRegister is stable and sturdy being
                         // no foul message can endanger him

  else
    inherited Perform(Message);
  end;
end;

function TModuleRegister.RegisterModule(Info: PModuleRegisterInfo): boolean;
var
  I: Long;

begin
  Result := false;
  if Info = nil then
    Exit;

  // check for duplicates
  Result := true;
  for I := 0 to ModuleInfos.Count - 1 do
    if Info^.ModuleInfo.Name =
      PModuleRegisterInfo(ModuleInfos[I])^.ModuleInfo.Name then
      Exit;

  ModuleInfos.Add(Info);
  Result := true;

  // notify module engines
  {TODO -o Galfar : create global msgs}
  for I := 0 to FModEngines.Count - 1 do
    TModEngine(FModEngines[I]).ProcessModulesChange;
end;

function TModuleRegister.ExternalInfo(Handle: THandle;
  Index: Long): PModuleRegisterInfo;
var
  Info: PModuleRegisterInfo;

begin
  Result := nil;
  if not IsValidModuleLib(Handle) then
    Exit;

  try
    New(Info);
    FillChar(Info^, SizeOf(Info^), 0);
    Info^.ModuleInfo := Objects.GetModuleInfo(Handle, Index);
    Info^.Handle := Handle;
    Info^.Index := Index;
    Result := Info;
  except
    RaiseError(ErrString(Self, 'RegisterExternal', 'Error when registering ' +
      'External Module Library "' + GetLibName(Handle) + '"!' + sLineBreak +
      'Make sure this file is really executable or library file.'),
      EEngine);
  end;
end;

function TModuleRegister.CreateModule(Info: PModuleRegisterInfo): IModule;
type
  TIntfFunction = function: IModule;
  TInstFunction = function: TModule;
  TClsFunction = function: TModuleClass;

var
  IntfFunction: TIntfFunction;
  InstFunction: TInstFunction;
  ClsFunction: TClsFunction;

begin
  Result := nil;
  if Info = nil then
    Exit;

  try
    case Info^.ModuleInfo.Style of
      msInternal:
        begin
          // ClassRef should be known here
          if Info^.Instance = nil then
            Info^.Instance := Info^.ClassRef.Create;
          if Info^.Intf = nil then
            Info^.Intf := Info^.Instance;
        end;
      msLowLevel:
        begin
          // first we need to find ClassRef of wrapper
          // for this low-level module.
          if Info^.ClassRef = nil then
            Info^.ClassRef := FindWrapperClassRef(Info);
          if Info^.ClassRef = nil then
            Exit;
          if Info^.Instance = nil then
            Info^.Instance := Info^.ClassRef.Create;
          if Info^.Intf = nil then
            Info^.Intf := Info^.Instance;

          // load low nmodule's props and functions to instance of wrapper
          (Info^.Intf as IExternalModule).LoadFromLib(Info^.Handle,
            Info^.Index);
        end;
      msInterface:
        if Info^.Intf = nil then
        begin
          // we first retrieve function which returns
          // created Interface
          @IntfFunction := GetModuleExtInfo(Info^.Handle, Info^.Index);
          Info^.Intf := IntfFunction;

          Info^.Instance := TModule(GetIntfImplementor(Info^.Intf));
          if Info^.Instance <> nil then
          begin
            // This is important line !!
            // because TModEngine compares Info^.Intf.Name = OtherInfo^.ModuleInfo.Name
            Info^.Instance.FInfo := Info^.ModuleInfo;
            Info^.ClassRef := TModuleClass(Info^.Instance.ClassType);
          end;
        end;
      msClassInst:
        if Info^.Instance = nil then
        begin
          // we first retrieve function which returns
          // created Instance of module
          @InstFunction := GetModuleExtInfo(Info^.Handle, Info^.Index);
          Info^.Instance := InstFunction;

          if Info^.Instance <> nil then
          begin
            Info^.Instance.FInfo := Info^.ModuleInfo;
            Info^.Intf := Info^.Instance;
            Info^.ClassRef := TModuleClass(Info^.Instance.ClassType);
          end;
        end;
      msClassRef:
        if Info^.ClassRef = nil then
        begin
          // we first retrieve function which returns
          // module's class reference
          @ClsFunction := GetModuleExtInfo(Info^.Handle, Info^.Index);
          Info^.ClassRef := ClsFunction;

          if Info^.ClassRef <> nil then
          begin
            Info^.Instance := Info^.ClassRef.Create;
            if Info^.Instance <> nil then
              Info^.Instance.FInfo := Info^.ModuleInfo;
            Info^.Intf := Info^.Instance;
          end;
        end;
    end;
    Result := Info^.Intf;
  except
    RaiseError(ErrString(Self, 'CreateModule',
      Format('Error when creating new module (Name: %s, Class: %s).',
      [Info^.ModuleInfo.Name, Iff(Info^.ClassRef <> nil,
        Info^.ClassRef.ClassName,
        '<none>')])), EEngine);
    Result := nil;
  end;
end;

procedure TModuleRegister.SearchExternals;
var
  Path: string;

begin
  Path := ExeDir + Sys.sModulesDirectory;
  SearchDirectory(Path);
  Path := ExeDir;
  SearchDirectory(Path);
  Path := ExeDir + Sys.sBinModulesDirectory;
  SearchDirectory(Path);
  Path := DirUp(ExeDir) + Sys.sBinModulesDirectory;
  SearchDirectory(Path);
end;

procedure TModuleRegister.SearchDirectory(const Path: string);
var
  Name: string;
  Rec: TSearchRec;
  Handle: THandle;
  Res, Index: Long;

begin
  Name := Path + PathDelim + sModuleFileMask;
  FillChar(Rec, SizeOf(Rec), 0);
  Res := FindFirst(Name, faAnyFile, Rec);
  try
    try
      while Res = 0 do
      begin
        Handle := 0;
        // Check if it is executable file.
        // If it is not Windows shows ugly error which can
        // not be handled by try..except
        if IsExecutable(Path + PathDelim+ Rec.Name) then
          // Note: if Rec.Name describes BPL package
          // this call initializes this package.
          // If another package is then loaded and it contains some
          // same named units Error will be raised and none of them
          // will be loaded corectly.
          // E.g. Delphi creates backup ModBpl.~bpl which contains same units
          // like ModBpl.bpl
          Handle := LoadLib(Path + PathDelim+ Rec.Name);
        if IsValidModuleLib(Handle) then
          for Index := 0 to GetModuleCount(Handle) - 1 do
            RegisterModule(ExternalInfo(Handle, Index))
        else
          UnLoadLib(Handle);
        Res := FindNext(Rec);
      end;
    except
      RaiseError(ErrString(Self, 'SearchDirectory', 'Error when loading ' +
        'possible External Module Library "' + Rec.Name + '"!' + sLineBreak +
        'Make sure this file is really executable or library file.'),
        EEngine);
    end;
  finally
    FindClose(Rec);
  end;
end;


{*************************************************************************}
{                             Other functions                             }
{*************************************************************************}


function ModuleRegisterInfo(Info: PModuleInfo; ClassRef: TModuleClass; Instance:
  TModule; Intf: IModule):
  PModuleRegisterInfo;
begin
  New(Result);
  Result.ModuleInfo := Info^;
  Result.ClassRef := ClassRef;
  Result.Instance := Instance;
  Result.Intf := Intf;
end;

procedure FreeModuleRegisterInfo(var Info: PModuleRegisterInfo);
begin
  if Info = nil then
    Exit;
  try
    Dispose(Info);
  except
  // don't show anything
  end;
  Info := nil;
end;

procedure RegisterIntModule(Info: PModuleInfo; ClassRef:
  TModuleClass);
var
  RInfo: PModuleRegisterInfo;
begin
  RInfo := ModuleRegisterInfo(Info, ClassRef, nil, nil);
  if not ModuleRegister.RegisterModule(RInfo) then
    FreeModuleRegisterInfo(RInfo);
end;

function GetDefaultIntModuleInfo: TModuleInfo;
begin
  Result := DefaultIntModuleInfo;
  StrPCopy(Result.LibName, ExtractFileName(ExeFullName));
  Result.Version := EngineVersion;
end;

function SameIDCodes(const ID1, ID2: TIDCode): boolean;
begin
  Result := CompareMem(@ID1, @ID2, SizeOf(TIDCode));
end;

function SameIDType(const IDType, IDCode: TIDCode): boolean;
begin
  Result := CompareMem(@IDType, @IDCode, 3 * SizeOf(Char));
end;

function ModuleStyleToStr(Style:TModuleStyle):string;
begin
 Result:=ModuleStyleNames[Style];
end;


{*************************************************************************}
{                       Some impostant functions                          }
{*************************************************************************}


procedure InitModuleRegister;
begin
  ModuleRegisterInstance := TModuleRegister.Create;
  ModuleRegister := ModuleRegisterInstance;
  ModuleRegister.SearchExternals;
end;

procedure FreeModuleRegister;
begin
  ReleaseIntf(ModuleRegister);
  ModuleRegisterInstance := nil;
end;


initialization

  InitModuleRegister;


finalization

  FreeModuleRegister;

end.

