{*************************************************************************}
{*   Strings                                                             *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Sys\Strings.pas                                                     *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 16th February 2003  (16.02.2003)                           }
{*************************************************************************}


{This unit is string manipulation functions library.}


unit Strings;


{TODO -o Glafar: Add replace/search function (FastStrings)}
{TODO -o Galfar : Rewrite StrTokenEnd to search tokens from the end of the string}


interface


uses
  Sys, Objects;


{*************************************************************************}
{                   AnsiString functions                                  }
{*************************************************************************}


{* String comparsions *}


{Compares two strings case-insensitive and returns <0 if
 S1<S2, >0 if S1>S2 or 0 if S1=S2}
function CompStr(const S1, S2: string): Long;

{Compares two strings case-sentive.
 Return is same as in Compare() }
function CompStrCase(const S1, S2: string): Long;

{Compares two strings case-insensitive}
function SameStr(const S1, S2: string): boolean;

{Compares two strings case-sensitive}
function SameStrCase(const S1, S2: string): boolean;


{* String Array/TStrings functions *}


{Returns true if S is in Arr, case-insensitive}
function StrInArray(const S: string; const Arr: array of string): boolean;

{Returns true if S is in Arr, case-sensitive}
function StrInArrayCase(const S: string; const Arr: array of string): boolean;

{Returns true if S is in TStrings, case-insensitive}
function StrInList(const S: string; List: TStrings): boolean;

{Returns true if S is in TStrings, case-insensitive}
function StrInListCase(const S: string; List: TStrings): boolean;

{Fills instance of TStrings filled with given strings.}
procedure StrArrayToList(const Arr: array of string;List:TStrings);


{* String split/pos/trim/... functions *}


{Splits S at pos of Separator and returns part before Sep. S is modified
after operation}
function StrToken(const S: string; Separator: Char): string; overload;

{Same as above, only Separators can be multiple}
function StrToken(const  S: string; const Seps: array of Char): string;overload;

{Splits S at pos of Separator and returns part after Sep}
function StrTokenEnd(const S: string; Separator: Char): string;overload;

{Same as above, only Separators can be multiple}
function StrTokenEnd(const S: string; const Seps: array of Char):string; overload;



implementation


{*************************************************************************}
{                   AnsiString functions                                  }
{*************************************************************************}


function CompStr(const S1, S2: string): Long;
asm
        PUSH    ESI
        PUSH    EDI
        PUSH    EBX
        MOV     ESI,EAX
        MOV     EDI,EDX
        OR      EAX,EAX
        JE      @@0
        MOV     EAX,[EAX-4]
@@0:    OR      EDX,EDX
        JE      @@1
        MOV     EDX,[EDX-4]
@@1:    MOV     ECX,EAX
        CMP     ECX,EDX
        JBE     @@2
        MOV     ECX,EDX
@@2:    CMP     ECX,ECX
@@3:    REPE    CMPSB
        JE      @@6
        MOV     BL,BYTE PTR [ESI-1]
        CMP     BL,'a'
        JB      @@4
        CMP     BL,'z'
        JA      @@4
        SUB     BL,20H
@@4:    MOV     BH,BYTE PTR [EDI-1]
        CMP     BH,'a'
        JB      @@5
        CMP     BH,'z'
        JA      @@5
        SUB     BH,20H
@@5:    CMP     BL,BH
        JE      @@3
        MOVZX   EAX,BL
        MOVZX   EDX,BH
@@6:    SUB     EAX,EDX
        POP     EBX
        POP     EDI
        POP     ESI
end;

function CompStrCase(const S1, S2: string): Long;
asm
        PUSH    ESI
        PUSH    EDI
        MOV     ESI,EAX
        MOV     EDI,EDX
        OR      EAX,EAX
        JE      @@1
        MOV     EAX,[EAX-4]
@@1:    OR      EDX,EDX
        JE      @@2
        MOV     EDX,[EDX-4]
@@2:    MOV     ECX,EAX
        CMP     ECX,EDX
        JBE     @@3
        MOV     ECX,EDX
@@3:    CMP     ECX,ECX
        REPE    CMPSB
        JE      @@4
        MOVZX   EAX,BYTE PTR [ESI-1]
        MOVZX   EDX,BYTE PTR [EDI-1]
@@4:    SUB     EAX,EDX
        POP     EDI
        POP     ESI
end;

function SameStr(const S1, S2: string): boolean;
asm
        CMP     EAX,EDX
        JZ      @1
        OR      EAX,EAX
        JZ      @2
        OR      EDX,EDX
        JZ      @3
        MOV     ECX,[EAX-4]
        CMP     ECX,[EDX-4]
        JNE     @3
        CALL    CompStr
        TEST    EAX,EAX
        JNZ     @3
@1:     MOV     AL,1
@2:     RET
@3:     XOR     EAX,EAX
end;

function SameStrCase(const S1, S2: string): boolean;
asm
        CMP     EAX,EDX
        JZ      @1
        OR      EAX,EAX
        JZ      @2
        OR      EDX,EDX
        JZ      @3
        MOV     ECX,[EAX-4]
        CMP     ECX,[EDX-4]
        JNE     @3
        CALL    CompStrCase
        TEST    EAX,EAX
        JNZ     @3
@1:     MOV     AL,1
@2:     RET
@3:     XOR     EAX,EAX
end;

function StrInArray(const S: string; const Arr: array of string): boolean;
var
  I: Long;
begin
  Result := false;
  for I := Low(Arr) to High(Arr) do
    if CompStr(S, Arr[I]) = 0 then
    begin
      Result := true;
      Exit;
    end;
end;

function StrInArrayCase(const S: string; const Arr: array of string): boolean;
var
  I: Long;
begin
  Result := false;
  for I := Low(Arr) to High(Arr) do
    if CompStrCase(S, Arr[I]) = 0 then
    begin
      Result := true;
      Exit;
    end;
end;

function StrInList(const S: string; List: TStrings): boolean;
var
  I: Long;
begin
  Result := false;
  for I := 0 to List.Count - 1 do
    if CompStr(S, List.Strings[I])=0 then
    begin
    Result := true;
    Exit;
    end;
end;

function StrInListCase(const S: string; List: TStrings): boolean;
var
  I: Long;
begin
  Result := false;
  for I := 0 to List.Count - 1 do
    if CompStrCase(S, List.Strings[I])=0 then
    begin
    Result := true;
    Exit;
    end;
end;

procedure StrArrayToList(const Arr: array of string;List:TStrings);
var
 I: Long;
begin
  List.Clear;
  for I := Low(Arr) to High(Arr) do
    List.Add(Arr[I]);
end;

function StrToken(const S: string; Separator: Char): string;
var
  I: Long;
begin
  I := Pos(Separator, S);
  if I <> 0 then
    Result := Copy(S, 1, I - 1)
  else
    Result := S;
end;

function StrToken(const S: string;const Seps: array of Char): string;
var
  I, J, K: Long;
begin
  J := Length(S) + 1;
  for I := Low(Seps) to High(Seps) do
  begin
    K := Pos(Seps[I], S);
    if K < J then
      J := K;
  end;
  if J <> Length(S) + 1 then
    Result := Copy(S, 1, J - 1)
  else
    Result := S;
end;

function StrTokenEnd(const S: string; Separator: Char): string;
var
  I: Long;
  SS: string;
begin
  SS := S;
  I := Pos(Separator, SS);
  if I <> 0 then
  begin
    Delete(SS, 1, I);
    Result := SS;
  end
  else
    Result := '';
end;

function StrTokenEnd(const S: string; const Seps: array of Char):string;
var
  I, J, K: Long;
  SS: string;
begin
  J := 0;
  for I := Low(Seps) to High(Seps) do
  begin
    K := Pos(Seps[I], S);
    if K > J then
      J := K;
  end;
  if J <> 0 then
  begin
    SS := S;
    Delete(SS, 1, J);
    Result := SS;
  end
  else
    Result := SS;
end;

end.

