{*************************************************************************}
{*   Sys                                                                 *}
{*   Copyright (C) 2003 Marek Mauder, Vampyre SoftWorks                  *}
{*   Sys\Sys.pas                                                         *}
{*************************************************************************}
{                                                                         }
{ This program is free software; you can redistribute it and/or modify it }
{ under the terms of the GNU General Public License as published by the   }
{ Free Software Foundation; either version 2 of the License, or (at your  }
{ option) any later version. This program is distributed in the hope that }
{ it will be useful, but WITHOUT ANY WARRANTY; without even the implied   }
{ warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See   }
{ the GNU General Public License for more details.                        }
{                                                                         }
{ You should have received a copy of the GNU General Public License       }
{ along with this program; if not, write to the Free Software Foundation, }
{ Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.         }
{                                                                         }
{*************************************************************************}
{  Created On: 16th February 2003  (16.02.2003)                           }
{*************************************************************************}


{This is system unit of entire engine. It contains base types,
functions, classes, ...
for memory allocation/freeing, error handling, library loading, messaging, ...}


{$M-}
{$J-}
{$A8}
{$X+}


unit Sys;


{DONE -o Galfar : Write our own Exception handler (System.ExceptProc)
with Stop Execution button (Wait for functional Kylix).}
{DONE -o Galfar : Add here Swap procedures/funcs from Maths}
{DONE -o Galfar : Store info about loaded libraries.}
{DONE -o Galfar : Add package info/... support.}
{DONE -o Galfar : Add support for search some other paths in LoadLibrary}
{TODO -o Galfar : Add some basic file IO functions.}
{DONE -o Galfar : Add Release proc for GetIntfImplementor(I).Free }
{DONE -o Galfar : Add raiseError proc instead of raise Exception.create.
 Add option to don't show error dialogs in raiseError}

{TODO -o Galfar: When creating TRenderer classes -
create TRenderer then descend to T2DRenderer (DDraw,GDI,d3d,ogl...)
and T3DRenderer (D3D, OGL)}

{TODO -o Galfar: Do not forget on D3DSpy in Dx9 SDK when creating
D3D9 rendereur}

{TODO -o Galfar : Warning! try ... except is not working well with
RaiseError. Exceptions are catched by first
... except  RaiseError() end;    in the way}

{DONE -o Galfar : Replace all GetMem/FreeMem operating on ^TRecord
with New/Dispose}
{TODO -o Galfar : Move some basic string functions here}
{DONE -o Galfar : Move SwapLong and other here from PngModule.pas}

{TODO -o Galfar : Do not forget to add SetEnvironmentString 'path'
for BPL modules}

{TODO -o Galfar : Remove all out parameters}


interface


uses
  SysUtils;


type

  {Some integer types}
  DWord = Cardinal;
  ULong = Cardinal;
  Small = SmallInt;
  Integer = LongInt;
  Short = ShortInt;
  Long = LongInt;

  {Some pointer types}
  PLong = ^Long;
  PShort = ^Short;
  PSmall = ^Small;
  PDWord = ^DWord;
  PULong = ^ULong;


  {Floating point types.
   To change precision use Double}
  Float = Single;
  //Float = Double;

  PFloat = ^Float;


  {String and character types}
  // string = AnsiString
  WString = WideString;
  WChar = WideChar;

  PString = ^string;
  PWString = ^WString;
  PWChar = ^WChar;


  {some other types}
  THandle = ULong;

  PInterface = ^IInterface;


const

  {Maximum values of some types not declared in System.pas}
  MaxLong = High(Long);
  MaxULong = High(ULong);
  MaxInt64 = High(Int64);

  {Minimum values of some types not declared in System.pas}
  MinLong = Low(Long);
  MinULong = Low(ULong);
  MinInt64 = Low(Int64);

  {Max size for some arrays}
  MaxByteArraySize = MaxLong;
  MaxWordArraySize = MaxLong div 2;
  MaxLongArraySize = MaxLong div 4;
  MaxInt64ArraySize = MaxLong div 8;


type

  {some general arrays}
  TByteArray = array[0..MaxByteArraySize - 1] of Byte;
  TWordArray = array[0..MaxWordArraySize - 1] of Word;
  TLongArray = array[0..MaxLongArraySize - 1] of Long;
  TULongArray = array[0..MaxLongArraySize - 1] of Ulong;
  TInt64Array = array[0..MaxInt64ArraySize - 1] of Int64;
  TPCharArray = array[0..MaxLongArraySize - 1] of PChar;
  TStringArray = array[0..MaxLongArraySize - 1] of string;
  TPointerArray = array[0..MaxLongArraySize - 1] of Pointer;

  PByteArray = ^TByteArray;
  PWordArray = ^TWordArray;
  PLongArray = ^TLongArray;
  PULongArray = ^TULongArray;
  PInt64Array = ^TInt64Array;
  PPCharArray = ^TPCharArray;
  PStringArray = ^TStringArray;
  PPointerArray = ^TPointerArray;


  {Version info record}
  TVersionInfo = packed record
    case Integer of
      0: (
        Major: Byte;
        Minor: Byte;
        Build: Word; );
      1: (
        Version: ULong);
  end;

  {(*}  // do not process by Source Formatter

  {Platform info . It is used e.g. by external modules to
  define for which platform it is designed}
  TPlatform = (
    pfUnknown = 0,      // uknown or unimportant
    pfWin32 = 1,        // 32bit Windows (for modules it means  "Win32 only")
    pfWin64 = 2,        // 64bit Windows
    pfLinux = 3,        // Linux  (for modules it means  "Linux only")
    pfIndependent = 4,  // e.g. for modules which are cross-platform
    pfForce32 = MaxInt  // to compile as 32bit interger (for compatibility)
    );

  {*)}

  {Error handling related types}

  TRaiseErrorProc = procedure(const Message: string; ExClass: ExceptClass =
    nil);

  TShowErrorProc = function(const Message: string): Long;

  TTerminateProc = procedure({??});

  {(*}  // do not process by Source Formatter

  {Global structure containing error handling functions.
  Can be set/get by Set/GetErrorHandler functions.
  Purpose of this struct is similar to Delphi's TMemoryManager
  struct. Example: You're creating Direct3D application
  and you want to show errors by your great GUI. So you
  call GetErrorHandler, change ShowError member to your
  show error procedure and then with SetErrorHandler set
  new handler.}
  TErrorHandler = record
    RaiseError: TRaiseErrorProc;  // Basic error proc which behaves according to
                                  // options set in TErrorHandlerOptions
                                  // (show error? terminate app? ...)
    ShowError: TShowErrorProc;    // Shows error messages
                                  // returns 0 if user
                                  // selects Stop execution, non-0 otherwise
    Terminate: TTerminateProc;    // This is called on abnormal program
                                  // termination e.g. when StopExecution
                                  // StopExecution member of Options is true.
                                  // You can put some Free/Dispose calls here
  end;


  {Global options which affect behaviour of error handler functions}
  TErrorHandlerOptions = record
    ShowMessage: boolean;         // shows message dialog with error info
    StopExecution: boolean;       // auto stop execution when error occurs
    AskToStop: boolean;           // ask user to stop execution
    RaiseException: boolean;      // raises regular Delphi exception which exits
                                  // to debugger if running from IDE
  end;


  {Type of executable/lib file}
  TCompiledFileType = (ftExe,     // *.exe
    ftPackage,                    // *.bpl
    ftLibrary,                    // *.dll
    ftUnknown);                   // ?????

  {Contains info about units contained in Delphi executable.}
  TDelphiExeInfo = record
    FileType: TCompiledFileType;
    ContainsCount: Long;        // number of units contained in executable
    RequiresCount: Long;        // number of required packages
    ContainsNames: array[0..256 - 1] of string; // names of contained units
    RequiresNames: array[0..128 - 1] of string; // names of required packages
  end;


  {Returned by GetLibInfo function, provides basic
  info about loaded library.}
  TLibInfo = record
    Name: string;                 // e.g. "D3D9.dll"
    FullName: string;             // e.g. "c:\winxp\system32\d3d9.dll"
    FileType: TCompiledFileType;  // see TCompiledFileType
    Handle: THandle;              // Handle to instance of loaded lib
  end;


  {Message structure used for comunication between objects and other
  purposes.}
  TMsgRec = packed record
    MsgID: ULong;            // Message ID constant. According to this ID
                             // procedures which performs this message
                             // access other memers of structure.
                             // E.g. Object receibes SM_FreeSender so
                             // Sender can not be nil !
    Sender: TObject;         // Identifies object that send this message
    case Long of
      0: (Data: Pointer);    // Custom pointer data
      1: (ULParam: ULong);   // Custom unsigned Long value
      2: (LParam: Long);     // Custom Long value
      3: (Words: array[0..1] of Word);  // Custom word values
      4: (Bytes: array[0..3] of Byte);  // Custom byte values
      5: (Chars: array[0..3] of Char);  // Custom chars
  end;


const

   {Sys.pas messages(0 - 127)
   All messages defined in Sys directory must begin with SM prefix.}
   SM_First =        00000;

   // .. add something really important at indexes 0,1

   SM_FreeSelf =     00002;     // Receiver calls Self.Free;
   SM_FreeSender =   00003;     // Receiver calls Sender.Free;
   SM_SetName =      00004;     // Sets name of object (TMsgRec.Data must
                                // contain pointer to valid string)

   // ...

   SM_Last =         01023;
   {*)}


const

   {Tolerance when comparing floats}
  FloatTolerance: Float = 0.000001;


type

  {Structs for accessing package info stored in Delphi compiled
  files. Taken from SysUtils' implementation section.}
  PPkgName = ^TPkgName;
  TPkgName = packed record
    HashCode: Byte;
    Name: array[0..255] of Char;
  end;

  PUnitName = ^TUnitName;
  TUnitName = packed record
    Flags: Byte;
    HashCode: Byte;
    Name: array[0..255] of Char;
  end;

  PPackageInfoHeader = ^TPackageInfoHeader;
  TPackageInfoHeader = packed record
    Flags: Cardinal;
    RequiresCount: Integer;
    {Requires: array[0..9999] of TPkgName;
    ContainsCount: Integer;
    Contains: array[0..9999] of TUnitName;}
  end;


{*************************************************************************}
{                       Classes/Interfaces                                }
{*************************************************************************}


type

  {Base interface for all engine interfaces.}
  IAbstract = interface(IInterface)
    ['{7453BA5A-B1E0-4CC6-A933-9640BE1501B2}']

    function GetName: string;
    function GetID: Long;
    procedure SetName(const Value: string);

    procedure Perform(var Message: TMsgRec);

    property Name: string read GetName write SetName;
    property ID: Long read GetID;

  end;


  {Base class for all engine objects.It is meant as abstract class
  so do not use it directly - use TBaseObject from Objects.pas instead.}
  TAbstractObject = class(TInterfacedObject, IAbstract)
  protected

    FName: string;
    FID: Long;

    function GetName: string; virtual;
    function GetID: Long; virtual;
    procedure SetName(const Value: string); virtual;

  public

    constructor Create; virtual;
    destructor Destroy; override;

    {Each object can receive messages and this procedure
    can process them.
    Be careful with sending messages like SM_Free, ...}
    procedure Perform(var Message: TMsgRec); virtual;

    {Overrides basic TObject's message dispatching method so
    it calls Perform. So that you can use Dispatch method instead of
    direct calling Perform (e.g. if you have list of Objects and
    you don't want to typecast every object}
    procedure DefaultHandler(var Message); override;


  published

    {Object's name. As default it is set to ClassName without begining T.}
    property Name: string read GetName write SetName;

    {Object's unique ID. It is set in constructor.}
    property ID: Long read GetID;


  end;


{*************************************************************************}
{                         Misc functions                                  }
{*************************************************************************}


{Switches boolean value}
procedure Switch(var Value: boolean);

{Swaps two values}
procedure Swap(var A, B: Long); overload;
procedure Swap(var A, B: Float); overload;
procedure Swap(var A, B: string); overload;
procedure Swap(var A, B: Pointer); overload;
procedure Swap(var A, B: boolean); overload;

{Equivalents to C "? :" operator.}
function Iff(const Condition: Boolean; const TruePart, FalsePart: string):
  string; overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Char): Char;
overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Byte): Byte;
overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Long): Long;
overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: DWord): DWord;
overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Float): Float;
overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Boolean):
  Boolean; overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Pointer):
  Pointer; overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Int64): Int64;
overload;
function Iff(const Condition: Boolean; const TruePart, FalsePart: Variant):
  Variant; overload;

{Swaps high and low bytes of 32 bit values.}
procedure SwapLong(P: PLong; Count: Long); overload;

{Swaps high and low bytes of 32 bit value and returns swapped
value.}
function SwapLong(Value: Long): Long; overload;


{*************************************************************************}
{                   Memory/Pointer Management                             }
{*************************************************************************}


{Frees object and sets reference to nil. Supports only TObject descendants!!}
procedure FreeAndNil(var Obj);

{Frees memory pointer and sets reference to nil. P must be
valid pointer!!!}
procedure FreeMemNil(var P);

{Calls GetMem and then fills memory with Value byte.}
procedure GetMemFill(var P; Size: Long; Value: Byte = 0);

{Returns size of memory referenced by APointer. Works only with Memory
managers based on original Delphi's Memory manager.
Note: Size of alocated memory for pointer is aligned the same
way as records.}
function SizeOfMem(const APointer: Pointer): Long;

{Functions which replaces standard pointer functions. They are
used by SysMemoryMgr.}
function SysGetMem(Size: Long): Pointer;
function SysFreeMem(P: Pointer): Long;
function SysReallocMem(P: Pointer; Size: Long): Pointer;

{Indicates how many times was GetMem called}
function GetMemCount: Long;

{Indicates how many times was FreeMem called}
function FreeMemCount: Long;

{Indicates how many times was ReallocMem called}
function ReallocMemCount: Long;

{Indicates how many bytes was allocated in GetMem calls}
function GetMemSize: Int64;

{Indicates how many bytes was deallocated in FreeMem calls}
function FreeMemSize: Int64;

{Indicates how many bytes was reallocated in ReallocMem calls}
function ReallocMemSize: Int64;


{*************************************************************************}
{                   Object/Class Functions                                }
{*************************************************************************}


{Returns number of objects created from program startup}
function GetCreatedObjects: Long;

{Returns number of objects freed from program startup}
function GetFreedObjects: Long;

{Returns number of "living" objects}
function GetLivingObjects: Long;

{Returns instance of class which implements given interface.
Interface I must not be nil!!
Use this only for Delphi interfaces (which uses TObject) - not for
e.g. DirectX COM objects}
function GetIntfImplementor(I: IInterface): TObject;

{Returns Reference count of given interface.
Use this only for Delphi interfaces (which uses TObject) -
not for e.g. DirectX COM objects}
function GetIntfRefCount(I: IInterface): Long;

{Releases implementor of given interface instantly,
ignoring reference count. I is set to nil.
I must be valid interface! Be careful with this!
Use this only for Delphi interfaces (which uses TObject) -
not for e.g. DirectX COM objects}
procedure ReleaseIntf(var I);


{*************************************************************************}
{                     Library/Package/File functions                      }
{*************************************************************************}


{Loads given library (can be any Win32/Linux executable) and returns Handle.
if AutoInitPackage = true and loaded library is Delphi package then
InitializePackage is called.
If operation was not successful Handle is 0.}
function LoadLib(const FileName: string; AutoInitPackage: boolean = true):
  THandle;

{Unloads given library.
if library is Delphi package and AutoFinalizePackage = true
then FinalizaPackage is called. Returns true if successful}
function UnloadLib(Handle: THandle; AutoFinalizePackage: boolean = true):
  boolean;

{Returns address of given symbol in given module. Returns nil if unsuccessful.}
function GetProcAddress(Handle: THandle; const Name: string): Pointer;

{Returns info about Delphi (must be compiled by Delphi or
C++ Builder) executable (exe/bpl/dll/so/...) contained units
and required packages. If exe was not compiled by Delphi/BCB then
returned struct is empty and result is false.}
function GetDelphiExeInfo(Handle: THandle; out Info: TDelphiExeInfo): boolean;

{Taken from SysUtils' implementation section this
function returns resource "PACKAGEINFO" strored in
all Delphi compiled files (exe/bpl/dll/...)}
function PackageInfoTable(Module: HMODULE): PPackageInfoHeader;

{Returns true if library identified by Handle is valid
Delphi package (BPL).}
function IsLibPackage(Handle: THandle): boolean;

{Returns number of records in LibInfo array (all files loaded
by LoadLib or already loaded (like runtime Packages)).}
function GetLibInfoCount: Long;

{Returns information of given library.
Index can be 0..GetLibInfoCount-1}
function GetLibInfo(Index: Long): TLibInfo;

{Returns path and name of loaded library defined by Handle.}
function GetLibName(Handle: THandle): string;

{Returns full path to running executable.}
function ExeFullName: string;

{Returns directory where running executable is located.}
function ExeDir: string;

{Returns dir parent of Directory, e.g. C:\Umajo\Bin -> C:\Umajo\}
function DirUp(const Directory: string): string;

{Returns true if file is Win32/Linux executable/library. If Path is
valid full path to file it checks header, otherwise checks extension.}
function IsExecutable(const FileName: string): boolean;

{Returns true if application was compiled with runtime packages.}
function IsCompiledWithPackages: boolean;


{*************************************************************************}
{                           Messaging functions                           }
{*************************************************************************}


{Creates valid  TMsgRec from given parameters.}
function CreateMsg(MsgID: ULong): TMsgRec; overload;

{Creates valid  TMsgRec from given parameters.}
function CreateMsg(MsgID: ULong; Sender: TObject): TMsgRec; overload;

{Creates valid  TMsgRec from given parameters.}
function CreateMsg(MsgID: ULong; Sender: TObject; Data: Pointer): TMsgRec;
overload;

{Creates valid  TMsgRec from given parameters.}
function CreateMsg(MsgID: ULong; Sender: TObject; ULongParam: ULong): TMsgRec;
overload;


{Creates valid  TMsgRec from given parameters.}
procedure CreateMsg(MsgID: ULong; out Msg: TMsgRec); overload;

{Creates valid  TMsgRec from given parameters.}
procedure CreateMsg(MsgID: ULong; Sender: TObject; out Msg: TMsgRec); overload;

{Creates valid  TMsgRec from given parameters.}
procedure CreateMsg(MsgID: ULong; Sender: TObject; Data: Pointer; out Msg:
  TMsgRec); overload;

{Creates valid  TMsgRec from given parameters.}
procedure CreateMsg(MsgID: ULong; Sender: TObject; ULongParam: ULong; out Msg:
  TMsgRec); overload;


{*************************************************************************}
{                             Base Math                                   }
{*************************************************************************}


{Returns -1 if Value is negative, otherwise returns 1 }
function Sign(Value: Long): Short; overload;
function Sign(Value: Float): Short; overload;

{Returns Min of two values}
function Min(A, B: Long): Long; overload;
function Min(A, B: Float): Float; overload;

{Returns Max of two values}
function Max(A, B: Long): Long; overload;
function Max(A, B: Float): Float; overload;

{Clamps value by Min and Max}
function Clamp(Value: Float; Min: Float = 0.0; Max: Float = 1.0): Float;
overload;
function Clamp(Value: Long; Min: Long = 0; Max: Long = 1): Long; overload;

{Compares two floats, returns true if A and B are (almost) identical.
Uses FloatTolerance constant in comparsion.}
function CompF(A, B: Float): boolean;

{Is float zero? Uses FloatTolerance constant in comparsion.}
function IsZeroF(Value: Float): boolean;

{Returns float   0 <= X < 1}
function Rand: Float; overload;

{Returns long 0 <= X < Max}
function Rand(Max: Long): Long; overload;
function Rand(Max: Float): Float; overload;


{*************************************************************************}
{                            Errors/Strings                               }
{*************************************************************************}


{Generates formatted error string containing Caller info, function name and
message. Use in exceptions and assertations. When called like this:
Assert(Umajo<>nil,ErrString('piko')) assert handler inserts
info about source file into generated message.}
function ErrString(Caller: TObject; const FuncName, Message: string):
  string;

{Returns string representatin of TVersionInfo.}
function VersionToStr(const Version: TVersionInfo): string;

{Returns string representatin of TPlatform.}
function PlatformToStr(Platform: TPlatform): string;

{Use this procedure instead of various
"raise Exception.Create". This proc shows error info according
to values of some variables (Store info into file? Show error dialog?
Raise exception?).
Usage: RaiseError(ErrString(Self,'LoadData','Error when loading data.'),
ELoadData)
This actually calls ErrorHandler.RaiseError.}
procedure RaiseError(const Message: string; ExClass: ExceptClass = nil);

{Sets new Error handling procedures}
procedure SetErrorHandler(const NewHandler: TErrorHandler);

{Gets current Error handling procedures}
procedure GetErrorHandler(var Handler: TErrorHandler);

{Default procs for ErrorHandler.}
procedure SysRaiseError(const Message: string; ExClass: ExceptClass = nil);
function SysShowError(const Message: string): Long;
procedure SysTerminate();


{*************************************************************************}
{                            Globals                                      }
{*************************************************************************}


const

  {Actual engine version. Read-only.}
  EngineVersion: TVersionInfo = (
    Major: 00;
    Minor: 01;
    // days from 15.2.2003
    Build: 0075; // 1.5.2003
    );


  {Actual platform on which engine was compiled.}
  Platform: TPlatform =
{$IFDEF WIN32}pfWin32{$ENDIF}
{$IFDEF LINUX}pfLinux{$ENDIF}
{$IFDEF WIN64}pfWin64{$ENDIF};


  {Base directory names}
  sBinDirectory = 'bin';
  sModulesDirectory = 'modules';
  sBinModulesDirectory = sBinDirectory + PathDelim + sModulesDirectory;
  sLogsDirectory = 'logs';
  sSettingsDirectory = 'settings';


var

  {Actual options for Error handling procedures.
  Defaults are set in initialization section.}
  ErrorHandlerOptions: TErrorHandlerOptions;


implementation


uses
{$IFDEF MSWINDOWS}
  Windows
{$ENDIF}
{$IFDEF LINUX}
  XLib, Libc, Qt
{$ENDIF}
  ;


type


  {Internal structure for storing info about loaded libraries.}
  TIntLibInfo = record
    Name: array[0..32 - 1] of Char; // without path, name with full path
    // can be obtained by Sys.GetLibName
    FileType: TCompiledFileType;
    Handle: THandle;
  end;

  TLibInfoArray = array[Word] of TIntLibInfo;

  PLibInfoArray = ^TLibInfoArray;


const

  {Our new memory manager which replaces System.pas' manager. It is set in init
  section of Sys.pas.}
  SysMemoryMgr: TMemoryManager = (
    GetMem: SysGetMem;
    FreeMem: SysFreeMem;
    ReallocMem: SysReallocMem);


  {Our default error handler. It is set in
  init section of Sys.pas.}
  SysErrorHandler: TErrorHandler = (
    RaiseError: SysRaiseError;
    ShowError: SysShowError;
    Terminate: SysTerminate);


  {Platform names}
  sUnknownPlatform = 'Unknown platform';
  sWin32Platform = 'Windows 32bit';
  sWin64Platform = 'Windows 64bit';
  sLinuxPlatform = 'Linux';
  sIndependentPlatform = 'Platform independent';


var

  {Std Delphi's Memory Manager}
  StdMemoryMgr: TMemoryManager;


  {Stores actual error handling procedures}
  ErrorHandler: TErrorHandler;


  {Memory usage/alloc stats variables}
  GetMemCnt: Long = 0;
  FreeMemCnt: Long = 0;
  ReallocMemCnt: Long = 0;

  GetMemSz: Int64 = 0;
  FreeMemSz: Int64 = 0;
  ReallocMemSz: Int64 = 0;


  {Object statistics}
  NowID: Long = 1;
  ObjCreated: Long = 0;
  ObjFreed: Long = 0;


  {Loaded libraries info.}
  LibInfoCount: Long;

  LibInfoArray: PLibInfoArray;


{*************************************************************************}
{                         Misc functions                                  }
{*************************************************************************}


{This function generates new IDs. It is very simple but why complicate it?.
However it will repeat numbers after creation of 2147k objects}
function GetNewID: Long;
begin
  Result := NowID;
  Inc(NowID);
end;

procedure Switch(var Value: boolean);
begin
  Value := not Value;
end;

procedure Swap(var A, B: Long);
var
  T: Long;
begin
  T := A;
  A := B;
  B := T;
end;

procedure Swap(var A, B: Float);
var
  T: Float;
begin
  T := A;
  A := B;
  B := T;
end;

procedure Swap(var A, B: string);
var
  T: string;
begin
  T := A;
  A := B;
  B := T;
end;

procedure Swap(var A, B: Pointer);
var
  T: Pointer;
begin
  T := A;
  A := B;
  B := T;
end;

procedure Swap(var A, B: boolean);
var
  T: boolean;
begin
  T := A;
  A := B;
  B := T;
end;

function Iff(const Condition: Boolean; const TruePart, FalsePart: string):
  string;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;

function Iff(const Condition: Boolean; const TruePart, FalsePart: Char): Char;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;

function Iff(const Condition: Boolean; const TruePart, FalsePart: Byte): Byte;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;

function Iff(const Condition: Boolean; const TruePart, FalsePart: Long): Long;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;

function Iff(const Condition: Boolean; const TruePart, FalsePart: DWord): DWord;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;

function Iff(const Condition: Boolean; const TruePart, FalsePart: Float): Float;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;

function Iff(const Condition: Boolean; const TruePart, FalsePart: Boolean):
  Boolean;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;

function Iff(const Condition: Boolean; const TruePart, FalsePart: Pointer):
  Pointer;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;

function Iff(const Condition: Boolean; const TruePart, FalsePart: Int64): Int64;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;

function Iff(const Condition: Boolean; const TruePart, FalsePart: Variant):
  Variant;
begin
  if Condition then
    Result := TruePart
  else
    Result := FalsePart;
end;

procedure SwapLong(P: PLong; Count: Long);
asm
@Lp:
              MOV ECX, [EAX]
              BSWAP ECX
              MOV [EAX], ECX
              ADD EAX, 4
              DEC EDX
              JNZ @Lp
end;

function SwapLong(Value: Long): Long;
asm
              BSWAP EAX
end;


{*************************************************************************}
{                   Memory/Pointer Management                             }
{*************************************************************************}


procedure FreeAndNil(var Obj);
var
  Temp: TObject;
begin
  Temp := TObject(Obj);
  Pointer(Obj) := nil;
  Temp.Free;
end;

procedure FreeMemNil(var P);
begin
  FreeMem(Pointer(P));
  Pointer(P) := nil;
end;

procedure GetMemFill(var P; Size: Long; Value: Byte);
begin
  GetMem(Pointer(P), Size);
  FillChar(Pointer(P)^, Size, Value);
end;

function SizeOfMem(const APointer: Pointer): Long;
type
  PUsed = ^TUsed;
  TUsed = record
    SizeFlags: Long;
  end;

const
  cThisUsedFlag = 2;
  cPrevFreeFlag = 1;
  cFillerFlag = Integer($80000000);
  cFlags = cThisUsedFlag or cPrevFreeFlag or cFillerFlag;

var
  U: PUsed;

begin
  Result := 0;
  try
    if APointer <> nil then
    begin
      U := APointer;
      U := PUsed(PChar(U) - SizeOf(TUsed));
      if (U.SizeFlags and cThisUsedFlag) <> 0 then
        Result := (U.SizeFlags) and (not cFlags - SizeOf(TUsed));
    end;
  except
    // do not show any exceptions to user

    // If you get exception at this point it means
    // that pointer you are trying to measure
    // is invalid. It was not allocated nor it is nil.
    // e.g:  var Data:Pointer;  begin SizeOfMem(Data);
  end;
end;

function GetMemCount: Long;
begin
  Result := GetMemCnt;
end;

function FreeMemCount: Long;
begin
  Result := FreeMemCnt;
end;

function ReallocMemCount: Long;
begin
  Result := ReallocMemCnt;
end;

function GetMemSize: Int64;
begin
  Result := GetMemSz;
end;

function FreeMemSize: Int64;
begin
  Result := FreeMemSz;
end;

function ReallocMemSize: Int64;
begin
  Result := ReallocMemSz;
end;

function SysGetMem(Size: Long): Pointer;
begin
  Result := nil;
  try
    Result := StdMemoryMgr.GetMem(Size);
    Inc(GetMemCnt);
    Inc(GetMemSz, Size);
  except
    // do not show any exceptions to user
  end;
end;

function SysFreeMem(P: Pointer): Long;
begin
  Result := -1;
  if P <> nil then
  begin
    try
      Inc(FreeMemSz, SizeOfMem(P));
      Result := StdMemoryMgr.FreeMem(P);
      Inc(FreeMemCnt);
    except
      // do not show any exceptions to user
    end;
  end
  else
    Result := -1;
end;

function SysReallocMem(P: Pointer; Size: Long): Pointer;
begin
  Result := nil;
  if P <> nil then
  begin
    try
      Result := StdMemoryMgr.ReallocMem(P, Size);
      Inc(ReallocMemCnt);
      Inc(ReallocMemSz, Size);
    except
      // do not show any exceptions to user
    end;
  end;
end;


{*************************************************************************}
{                   Object/Class/Interface Functions                      }
{*************************************************************************}


function GetCreatedObjects: Long;
begin
  Result := ObjCreated;
end;

function GetFreedObjects: Long;
begin
  Result := ObjFreed;
end;

function GetLivingObjects: Long;
begin
  Result := Abs(ObjCreated - ObjFreed);
end;

function GetVirtualMethodCount(AClass: TClass): Long;
var
  BeginVMT: Long;
  EndVMT: Long;
  TablePointer: Long;
  I: Long;
begin
  BeginVMT := Long(AClass);
  EndVMT := PLong(Long(AClass) + vmtClassName)^;
  I := vmtSelfPtr + SizeOf(Pointer);
  repeat
    TablePointer := PLong(Long(AClass) + I)^;
    if (TablePointer <> 0) and (TablePointer >= BeginVMT) and
      (TablePointer < EndVMT) then
      EndVMT := Long(TablePointer);
    Inc(I, SizeOf(Pointer));
  until I >= vmtClassName;
  Result := (EndVMT - BeginVMT) div SizeOf(Pointer);
end;

function GetVirtualMethod(AClass: TClass; Index: Long): Pointer;
begin
  Result := PPointer(Integer(AClass) + Index * SizeOf(Pointer))^;
end;

function GetIntfImplementor(I: IInterface): TObject;
const
  AddByte = $04244483; // opcode for ADD DWORD PTR [ESP+4], Shortint
  AddLong = $04244481; // opcode for ADD DWORD PTR [ESP+4], Longint
type
  PAdjustSelfThunk = ^TAdjustSelfThunk;
  TAdjustSelfThunk = packed record
    case AddInstruction: LongInt of
      AddByte: (AdjustmentByte: ShortInt);
      AddLong: (AdjustmentLong: LongInt);
  end;
  PInterfaceMT = ^TInterfaceMT;
  TInterfaceMT = packed record
    QueryInterfaceThunk: PAdjustSelfThunk;
  end;
  TInterfaceRef = ^PInterfaceMT;
var
  QueryInterfaceThunk: PAdjustSelfThunk;
begin
  try
    Result := Pointer(I);
    if Assigned(Result) then
    begin
      QueryInterfaceThunk := TInterfaceRef(I)^.QueryInterfaceThunk;
      case QueryInterfaceThunk.AddInstruction of
        AddByte:
          Inc(PChar(Result), QueryInterfaceThunk.AdjustmentByte);
        AddLong:
          Inc(PChar(Result), QueryInterfaceThunk.AdjustmentLong);
      else
        Result := nil;
      end;
    end;
  except
    Result := nil;
  end;
end;

function GetIntfRefCount(I: IInterface): Long;
begin
  Result := 0;
  try
    // we must subtract one here because I parameter added one to
    // RefCount
    Result := TInterfacedObject(GetIntfImplementor(I)).RefCount - 1;
  except
    // don't show
  end;
end;

procedure ReleaseIntf(var I);
var
  Obj: TInterfacedObject;
begin
  Obj := nil;
  try
    Obj := TInterfacedObject(GetIntfImplementor(IInterface(I)));
  finally
    if Obj <> nil then
      // if refcount=1 then Delphi automatically releases interface
      // at line IIntf(I):=nil;
      if Obj.RefCount > 1 then
        Obj.Free;
  end;
  IInterface(I) := nil;
end;


{*************************************************************************}
{                       TAbstractObject Methods                           }
{*************************************************************************}


constructor TAbstractObject.Create;
begin
  inherited Create;
  Name := Copy(ClassName, 2, Length(ClassName) - 1);
  FID := GetNewID;
  Inc(ObjCreated);
end;

destructor TAbstractObject.Destroy;
begin
  Inc(ObjFreed);
  inherited Destroy;
end;

function TAbstractObject.GetName: string;
begin
  Result := FName;
end;

function TAbstractObject.GetID: Long;
begin
  Result := FID;
end;

procedure TAbstractObject.SetName(const Value: string);
begin
  FName := Value;
end;

procedure TAbstractObject.DefaultHandler(var Message);
begin
  inherited DefaultHandler(Message);
  Perform(TMsgRec(Message));
end;

procedure TAbstractObject.Perform(var Message: TMsgRec);
begin
  case Message.MsgID of
    SM_FreeSelf: Self.Free;
    SM_FreeSender: Message.Sender.Free;
    SM_SetName: SetName(PString(Message.Data)^);
  else
   // in descendants put here this line
   // inherited Perform(..);
   // to perform messages not processed by this handler
  end;
end;


{*************************************************************************}
{                       Library/Package functions                         }
{*************************************************************************}


// This function is from Strings.pas but we don't want to be dependent on it
function StrInArray(const S: string; const Arr: array of string): boolean;
var
  I: Long;
begin
  Result := false;
  for I := Low(Arr) to High(Arr) do
    if SysUtils.SameText(S, Arr[I]) then
    begin
      Result := true;
      Exit;
    end;
end;

{Internal function which adds new info record to LibInfoArray.}
procedure AddLibInfo(Handle: THandle);
var
  I: Long;
  Info: TDelphiExeInfo;

begin
  if Handle = 0 then
    Exit;

  // check for duplicates
  for I := 0 to LibInfoCount - 1 do
    if LibInfoArray[I].Handle = Handle then
      Exit;

  Inc(LibInfoCount);
  try
    ReallocMem(LibInfoArray, LibInfoCount * SizeOf(TIntLibInfo));

    StrPCopy(LibInfoArray[LibInfoCount - 1].Name,
      ExtractFileName(GetLibName(Handle)));
    LibInfoArray[LibInfoCount - 1].Handle := Handle;

    GetDelphiExeInfo(Handle, Info);
    LibInfoArray[LibInfoCount - 1].FileType := Info.FileType;

    if LibInfoArray[LibInfoCount - 1].FileType = ftUnknown then
    begin
      if StrInArray(ExtractFileExt(LibInfoArray[LibInfoCount - 1].Name),
        ['.exe', '']) then
        LibInfoArray[LibInfoCount - 1].FileType := ftExe;
      if StrInArray(ExtractFileExt(LibInfoArray[LibInfoCount - 1].Name),
        ['.bpl', '.dcl']) then
        LibInfoArray[LibInfoCount - 1].FileType := ftPackage;
      if StrInArray(ExtractFileExt(LibInfoArray[LibInfoCount - 1].Name),
        ['.dll', '.drv', '.vxd', '.so']) then
        LibInfoArray[LibInfoCount - 1].FileType := ftLibrary;
    end;

  except
    Dec(LibInfoCount);
    ReallocMem(LibInfoArray, LibInfoCount * SizeOf(TIntLibInfo));
  end;

end;

{Internal function which removess info record from LibInfoArray.}
procedure RemoveLibInfo(Handle: THandle);
var
  I, J: Long;
begin
  for I := 0 to LibInfoCount - 1 do
    if LibInfoArray[I].Handle = Handle then
      Break;

  for J := I + 1 to LibInfoCount - 1 do
    LibInfoArray[J - 1] := LibInfoArray[J];

  Dec(LibInfoCount);
  ReallocMem(LibInfoArray, LibInfoCount * SizeOf(TIntLibInfo));
end;

{This is actually wrapper to Win32/Linux LoadLibrary/dlopen.}
function LoadLib(const FileName: string; AutoInitPackage: boolean): THandle;
var
  Name: string;
begin
  Name := FileName;

  {if FileName is not path to exact file try some default
  locations first (Bin,Modules,Exedir,DirUp, ...). If file is not found here
  then let Win/Linux find file on their search paths}
  if not FileExists(FileName) then
  begin
    if FileExists(ExeDir + FileName) then
      Name := ExeDir + FileName;
    if FileExists(ExeDir + sBinDirectory + PathDelim + FileName) then
      Name := ExeDir + sBinDirectory + PathDelim + FileName;
    if FileExists(ExeDir + sModulesDirectory + PathDelim + FileName) then
      Name := ExeDir + sModulesDirectory + PathDelim + FileName;
    if FileExists(ExeDir + sBinModulesDirectory + PathDelim + FileName) then
      Name := ExeDir + sBinModulesDirectory + PathDelim + FileName;
    if FileExists(DirUp(ExeDir) + FileName) then
      Name := DirUp(ExeDir) + FileName;
    if FileExists(DirUp(DirUp(ExeDir)) + FileName) then
      Name := DirUp(DirUp(ExeDir)) + FileName;
  end;

{$IFDEF MSWINDOWS}
  Result := Windows.LoadLibrary(PChar(Name));
{$ENDIF}
{$IFDEF LINUX}
  Result := THandle(dlopen(PChar(Name), RTLD_LAZY));
{$ENDIF}

  // if it is package then initialize it
  if AutoInitPackage and IsLibPackage(Result) then
    InitializePackage(Result);

  // add to LibInfoArray
  AddLibInfo(Result);
end;

{This is actually wrapper to Win32/Linux FreeLibrary/dlclose.}
function UnloadLib(Handle: THandle; AutoFinalizePackage: boolean): boolean;
begin
  Result := false;
  if (Handle = 0) or (Handle = HInstance)
    or (Handle = MainInstance) then
    Exit;

  // if it is package then finalize it
  if AutoFinalizePackage and IsLibPackage(Handle) then
    FinalizePackage(Handle);

  RemoveLibInfo(Handle);

{$IFDEF MSWINDOWS}
  Result := Windows.FreeLibrary(Handle);
{$ENDIF}
{$IFDEF LINUX}
  Result := LongBool(dlclose(Pointer(Handle)));
  InvalidateModuleCache;
{$ENDIF}
end;

{This is actually wrapper to Windows/SysUtils GetProcAddress.}
function GetProcAddress(Handle: THandle; const Name: string): Pointer;
begin
{$IFDEF MSWINDOWS}
  Result := Windows.GetProcAddress(Handle, PChar(Name));
{$ENDIF}
{$IFDEF LINUX}
  Result := SysUtils.GetProcAddress(Handle, PChar(Name));
{$ENDIF}
end;

function PackageInfoTable(Module: HMODULE): PPackageInfoHeader;
var
  ResInfo: HRSRC;
  Data: THandle;
begin
  Result := nil;
  ResInfo := FindResource(Module, 'PACKAGEINFO', RT_RCDATA);
  if ResInfo <> 0 then
  begin
    Data := LoadResource(Module, ResInfo);
    if Data <> 0 then
    try
      Result := LockResource(Data);
      UnlockResource(Data);
    finally
      FreeResource(Data);
    end;
  end;
end;

function GetDelphiExeInfo(Handle: THandle; out Info: TDelphiExeInfo): boolean;
var
  InfoTable: PPackageInfoHeader;
  I: Long;
  PkgName: PPkgName;
  UName: PUnitName;
  Count: Long;
  Flgs: ULong;
begin
  Result := false;
  Info.ContainsCount := 0;
  Info.RequiresCount := 0;
  Info.FileType := ftUnknown;
  InfoTable := PackageInfoTable(Handle);
  if InfoTable = nil then
    Exit;
  with InfoTable^ do
  begin
    PkgName := PPkgName(Long(InfoTable) + SizeOf(InfoTable^));
    Info.RequiresCount := RequiresCount;
    for I := 0 to Info.RequiresCount - 1 do
    begin
      Info.RequiresNames[I] := PkgName.Name;
      Inc(Long(PkgName), StrLen(PkgName.Name) + 2);
    end;
    Count := Long(Pointer(PkgName)^);
    UName := PUnitName(Long(PkgName) + 4);
    Info.ContainsCount := Count;
    for I := 0 to Info.ContainsCount - 1 do
    begin
      Info.ContainsNames[I] := UName.Name;
      Inc(Long(UName), StrLen(UName.Name) + 3);
    end;
  end;
  Flgs := (InfoTable.Flags shr 30);
  if Flgs = 3 then
    Flgs := 0
  else
    if Flgs = 0 then
      Flgs := 3;
  Info.FileType := TCompiledFileType(Flgs);
  Result := true;
end;

function IsLibPackage(Handle: THandle): boolean;
begin
  Result := false;
  if Handle = 0 then
    Exit;

  Result := (GetProcAddress(Handle, 'Initialize') <> nil) and
    (GetProcAddress(Handle, 'Finalize') <> nil);
end;

function GetLibInfoCount: Long;
begin
  Result := LibInfoCount;
end;
     
function GetLibInfo(Index: Long): TLibInfo;
begin
  if (Index < 0) or (Index > LibInfoCount - 1) then
    Exit;

  try
    Result.Name := LibInfoArray[Index].Name;
    Result.Handle := LibInfoArray[Index].Handle;
    Result.FullName := GetModuleName(Result.Handle);
    Result.FileType := LibInfoArray[Index].FileType;
  except
    // do not show any info to user
  end;
end;

function GetLibName(Handle: THandle): string;
var
{$IFDEF LINUX}
  FileName: array[0..FILENAME_MAX] of Char;
{$ENDIF}
{$IFDEF MSWINDOWS}
  FileName: array[0..MAX_PATH] of Char;
{$ENDIF}
begin
  SetString(Result, FileName,
    GetModuleFileName(Handle, FileName, SizeOf(FileName)));
end;

function ExeFullName: string;
begin
  Result := GetLibName(MainInstance);
end;

function ExeDir: string;
begin
  Result := ExtractFileDir(ExeFullName) + PathDelim;
end;

function DirUp(const Directory: string): string;
var
  P: Long;
  S: string;
begin
  Result := Directory;
  if (Directory = PathDelim) then
    Exit;

  if (Pos(PathDelim, Result) = 0) then
  begin
    Result := Result + PathDelim;
    Exit;
  end;

  while Result[Length(Result)] = PathDelim do
  begin
    Delete(Result, Length(Result), 1);
    if Result = '' then
      Break;
  end;

  S := Result;
  P := Pos(PathDelim, S);
  if P = 0 then
  begin
    Result := Result + PathDelim;
    Exit;
  end;

  while P > 0 do
  begin
    Delete(S, 1, P);
    P := Pos(PathDelim, S);
  end;
  Delete(Result, Pos(S, Result), Length(S));
end;

function IsExecutable(const FileName: string): boolean;
const
{$IFDEF MSWINDOWS}
  ExeID = 'MZ';
{$ENDIF}
{$IFDEF LINUX}
  ExeID = #$7F'ELF';
{$ENDIF}

var
  Handle: Long;
  Ext: string;
{$IFDEF MSWINDOWS}
  Buffer: array[0..1] of Char;
{$ENDIF}
{$IFDEF LINUX}
  Buffer: array[0..3] of Char;
{$ENDIF}

begin
  Result := false;
  FillChar(Buffer, SizeOf(Buffer), 0);
  if FileExists(FileName) then
  try
    Handle := FileOpen(FileName, fmOpenRead or fmShareDenyNone);
    FileRead(Handle, Buffer, SizeOf(Buffer));
    FileClose(Handle);
    if Buffer = ExeID then
      Result := true;
  except
    // don't show anything
  end
  else
  begin
    Ext := ExtractFileExt(FileName);
    if StrInArray(Ext, ['.dll', '.exe', '.bpl', '', '.so', '.exm', '.vem']) then
      Result := true;
  end;
end;

function IsCompiledWithPackages: boolean;
begin
  Result := FindClassHInstance(System.TObject) <> HInstance;
end;


{*************************************************************************}
{                           Messaging functions                           }
{*************************************************************************}


function CreateMsg(MsgID: ULong): TMsgRec;
begin
  Result.MsgID := MsgID;
  Result.Sender := nil;
  Result.ULParam := 0;
end;

function CreateMsg(MsgID: ULong; Sender: TObject): TMsgRec;
begin
  Result.MsgID := MsgID;
  Result.Sender := Sender;
  Result.ULParam := 0;
end;

function CreateMsg(MsgID: ULong; Sender: TObject; Data: Pointer): TMsgRec;
begin
  Result.MsgID := MsgID;
  Result.Sender := Sender;
  Result.Data := Data;
end;

function CreateMsg(MsgID: ULong; Sender: TObject; ULongParam: ULong): TMsgRec;
begin
  Result.MsgID := MsgID;
  Result.Sender := Sender;
  Result.ULParam := ULongParam;
end;


procedure CreateMsg(MsgID: ULong; out Msg: TMsgRec);
begin
  Msg.MsgID := MsgID;
  Msg.Sender := nil;
  Msg.ULParam := 0;
end;

procedure CreateMsg(MsgID: ULong; Sender: TObject; out Msg: TMsgRec);
begin
  Msg.MsgID := MsgID;
  Msg.Sender := Sender;
  Msg.ULParam := 0;
end;

procedure CreateMsg(MsgID: ULong; Sender: TObject; Data: Pointer; out Msg:
  TMsgRec);
begin
  Msg.MsgID := MsgID;
  Msg.Sender := Sender;
  Msg.Data := Data;
end;

procedure CreateMsg(MsgID: ULong; Sender: TObject; ULongParam: ULong; out Msg:
  TMsgRec);
begin
  Msg.MsgID := MsgID;
  Msg.Sender := Sender;
  Msg.ULParam := ULongParam;
end;


{*************************************************************************}
{                             Base Math                                   }
{*************************************************************************}


function Sign(Value: Long): Short;
begin
  if Value < 0 then
    Result := -1
  else
    Result := 1;
end;

function Sign(Value: Float): Short;
begin
  if Value < 0 then
    Result := -1
  else
    Result := 1;
end;

function Min(A, B: Long): Long;
begin
  if A < B then
    Result := A
  else
    Result := B;
end;

function Min(A, B: Float): Float;
begin
  if A < B then
    Result := A
  else
    Result := B;
end;

function Max(A, B: Long): Long;
begin
  if A > B then
    Result := A
  else
    Result := B;
end;

function Max(A, B: Float): Float;
begin
  if A > B then
    Result := A
  else
    Result := B;
end;

function Clamp(Value: Float; Min: Float; Max: Float):
  Float;
begin
  if Value < Min then
    Result := Min
  else
    if Value > Max then
      Result := Max
    else
      Result := Value;
end;

function Clamp(Value: Long; Min: Long; Max: Long):
  Long;
begin
  if Value < Min then
    Result := Min
  else
    if Value > Max then
      Result := Max
    else
      Result := Value;
end;

function CompF(A, B: Float): boolean;
begin
  Result := (A = B) or (Abs(B - A) < FloatTolerance);
end;

function IsZeroF(Value: Float): boolean;
begin
  Result := Abs(Value) < FloatTolerance;
end;

function Rand: Float;
begin
  Result := Random;
end;

function Rand(Max: Long): Long;
begin
  Result := Random(Max);
end;

function Rand(Max: Float): Float;
begin
  Result := Random(Round(Max));
end;


{*************************************************************************}
{                            Errors/Strings                               }
{*************************************************************************}


function ErrString(Caller: TObject; const FuncName, Message: string):
  string;
var
  S: string;
begin
  if Caller = nil then
    S := '<none>'
  else
    if Caller is TAbstractObject then
      S := Format('Class: %s; Name: %s; ID: %d; Address: $%.8x',
        [Caller.ClassName, TAbstractObject(Caller).Name,
        TAbstractObject(Caller).ID, Cardinal(Caller)])
    else
      S := Format('Class: %s; Address: $%.8x',
        [Caller.ClassName, Cardinal(Caller)]);

  Result := sLineBreak + 'Error occurred!' + sLineBreak + 'Caller Object: ' + S
    + sLineBreak
    + 'Function: ' + FuncName + sLineBreak + 'Message:' +
    '                                                ' + sLineBreak + Message +
    sLineBreak + sLineBreak;
end;

function VersionToStr(const Version: TVersionInfo): string;
begin
  Result := Format('%.2d.%.2d.%.4d', [Version.Major, Version.Minor,
    Version.Build]);
end;

function PlatformToStr(Platform: TPlatform): string;
begin
  case Platform of
    pfUnknown: Result := sUnknownPlatform;
    pfWin32: Result := sWin32Platform;
    pfWin64: Result := sWin64Platform;
    pfLinux: Result := sLinuxPlatform;
    pfIndependent: Result := sIndependentPlatform;
  end;
end;

procedure RaiseError(const Message: string; ExClass: ExceptClass = nil);
begin
  if @ErrorHandler.RaiseError <> nil then
    ErrorHandler.RaiseError(Message, ExClass);
end;

function ShowError(const Message: string): Long;
begin
  Result := 1;
  if @ErrorHandler.ShowError <> nil then
    Result := ErrorHandler.ShowError(Message);
end;

procedure Terminate({??});
begin
  if @ErrorHandler.Terminate <> nil then
    ErrorHandler.Terminate();
end;

procedure SetErrorHandler(const NewHandler: TErrorHandler);
begin
  ErrorHandler := NewHandler;
end;

procedure GetErrorHandler(var Handler: TErrorHandler);
begin
  Handler := ErrorHandler;
end;

procedure SysRaiseError(const Message: string; ExClass: ExceptClass = nil);
var
  P: Long;
  Msg: string;

begin
  Msg := Message;

  if ErrorHandlerOptions.RaiseException then
    if ExClass <> nil then
      raise ExClass.Create(Msg)
    else
      raise Exception.Create(Msg);

  P := Pos('Message', Msg);
  // add info about exception class
  if (P > 0) and (ExClass <> nil) then
    Insert('Exception Class: ' + ExClass.ClassName + sLineBreak, Msg, P);

  if ErrorHandlerOptions.ShowMessage then
    case ShowError(Msg) of
      0: Terminate;
    end;

  if ErrorHandlerOptions.StopExecution then
    Terminate;
end;

function SysShowError(const Message: string): Long;
var
  MRes, Flags: ULong;
  S: string;

begin
  Result := 1;
{$IFDEF MSWINDOWS}
  if IsConsole then
  begin
    Writeln(sLineBreak + 'Application Error!');
    Write(Message);

    if ErrorHandlerOptions.AskToStop and
      (not ErrorHandlerOptions.StopExecution) then
    begin
      Writeln('Press [Y] to continue or [N] to stop execution. Then press ' +
        '[Enter] to accept.');
      Readln(S);
      if Pos('N', StrUpper(PChar(S))) > 0 then
        Result := 0;
    end
    else
      if (not ErrorHandlerOptions.AskToStop) and
        (not ErrorHandlerOptions.StopExecution) then
      begin
        Writeln('Press [Enter] to continue execution.');
        Readln;
      end
      else
        if ErrorHandlerOptions.StopExecution then
        begin
          Writeln('Press [Enter] to stop execution.');
          Readln;
        end;
  end

  else

  begin
    Flags := MB_ICONERROR;
    if ErrorHandlerOptions.AskToStop and
      (not ErrorHandlerOptions.StopExecution) then
    begin
      Flags := Flags or MB_YESNO;
      S := sLineBreak + 'Continue execution?' + sLineBreak;
    end
    else
    begin
      Flags := Flags or MB_OK;
      S := sLineBreak + 'Click OK to continue execution.' + sLineBreak;
    end;

    if ErrorHandlerOptions.StopExecution then
      S := sLineBreak + 'Click OK to stop execution.' + sLineBreak;

    MRes := Windows.MessageBox(GetForegroundWindow, PChar(Message + S),
      'Application Error', Flags);

    if MRes = IDNO then
      Result := 0;

  end;
{$ENDIF}

  {TODO -o Galfar : Add code here}
{$IFDEF LINUX}




{$ENDIF}
end;

procedure SysTerminate();
begin
  Halt(1);
end;


{*************************************************************************}
{                         Some important functions                        }
{*************************************************************************}


{Sets default values for various options}
procedure SetDefaultSettings;
begin
  {ErrorHandler options defaults}
  ErrorHandlerOptions.ShowMessage := true;
  ErrorHandlerOptions.StopExecution := false;
  ErrorHandlerOptions.AskToStop := true;
  ErrorHandlerOptions.RaiseException := false;
end;

procedure SetSysMemMgr;
begin
  GetMemoryManager(StdMemoryMgr);
  SetMemoryManager(SysMemoryMgr);
end;

procedure SetSysErrorHandler;
begin
  SetErrorHandler(SysErrorHandler);
end;

procedure InitLibInfo;
var
  CurModule: PLibModule;
begin
  LibInfoCount := 1;

  ReallocMem(LibInfoArray, SizeOf(TIntLibInfo));
  StrPCopy(LibInfoArray[0].Name, ExtractFileName(ExeFullName));
  LibInfoArray[0].FileType := ftExe;
  LibInfoArray[0].Handle := MainInstance;

  // add libraries from Systems' LibModuleList
  CurModule := LibModuleList;
  while CurModule <> nil do
  begin
    AddLibInfo(CurModule.Instance);
    CurModule := CurModule.Next;
  end;

  {DONE -o Galfar : Search trough LibModuleList}
  {TODO -o Galfar : Find the way to retrieve info about all
  libs loaded by process}
  {DONE -o Galfar : Add to LoadLibrary - check for duplicates}
  {DONE -o Galfar : Add to UnLoadLibrary }
end;

procedure FreeLibInfo;
begin
  LibInfoCount := 0;
  FreeMem(LibInfoArray);
end;

procedure AssertHandler(const Message, Filename: string;
  LineNumber: Long; ErrorAddr: Pointer);
var
  P: Long;
  Msg: string;
begin
  Msg := Message;
  //checks if Message was created by ErrString and if yes then insert source
  // info
  P := Pos('Message', Msg);
  if P > 0 then
    Insert('Source module: ' + FileName + sLineBreak +
      Format('Source line: %d' + sLineBreak + 'Error Address: $%.8x' +
      sLineBreak,
      [LineNumber, Cardinal(ErrorAddr)]), Msg, P)
    // create default msg - looks like std Delphi assert error message
  else
    Msg := Message + ' (' + FileName + ', line ' + IntToStr(LineNumber) + ')';

  RaiseError(Msg, EAssertionFailed);
end;



initialization

  SetSysMemMgr;
  SetSysErrorHandler;
  SetDefaultSettings;
  InitLibInfo;

  {Sets AssertErrorProc variable from System.pas to
  new handler (default is in SysUtils.pas - it causes exception
  and invoke debugger if running in Delphi IDE but
  shows ugly windows Send Error Message and Runtime Error
  dialogs if running off IDE}
  AssertErrorProc := @AssertHandler;


finalization

  FreeLibInfo;

end.

