{This is example of high-level module.
It is high level version of NotModule from low-level
examples.

It contains all three types of external
high-level modules = Interface, ClassInstance, ClassReference

Try load this module by Module Informator in \Demos
directory.}

library ModExHigh;


uses
  Sys,
  Storage,
  Objects,
  SysUtils,
  Windows;

{$R *.res}

{*************************************************************************}
{                        Exported functions                               }
{*************************************************************************}


procedure ModuleCount(var Count: Long); cdecl;
begin
  {Return valid module count. }
  Count := 3;
end;


procedure ModuleInfo(Index: Long; var Info: TModuleInfo); cdecl;
begin
  {Return valid info for high-level module.}
  {This info is for module No. 0}
  FillChar(Info, SizeOf(Info), 0);
  Info.Name := 'Not High Intf Module';
  Info.Description := 'Negates given memory.';
  StrPCopy(Info.LibName, ExtractFileName(GetLibName(HInstance)));
  Info.Creator := 'Marek Mauder, Vampyr';
  Info.Language := 'Delphi 7';
  Info.IDCode := 'CMP-NOT';
  Info.Style := msInterface;
  Info.Platform := pfIndependent;
  Info.Version.Version := $00300100;

  {Changes for module No. 1}
  if Index = 1 then
  begin
    Info.Name := 'Not High ClsInst Module';
    Info.Style := msClassInst;
  end;

  {Changes for module No. 2}
  if Index = 2 then
  begin
    Info.Name := 'Not High ClsRef Module';
    Info.Style := msClassRef;
  end;
end;


type

  {Declare new class for our module, overriding Compress and
  Decompress functions}
  TNotModule = class(TCompressModule)
  public
    function CompressBuffer(InData: Pointer; InSize: Long; var OutData: Pointer;
      var OutSize: Long): boolean; override;
    function DecompressBuffer(InData: Pointer; InSize: Long; var OutData:
      Pointer; var OutSize: Long): boolean; override;
  end;


{Function returning created interface}
function CreateIntf: IModule;
begin
  Result := TNotModule.Create;
end;

{Function returning created instance}
function CreateInst: TModule;
begin
  Result := TNotModule.Create;
end;

{Function returning class reference}
function CreateRef: TModuleClass;
begin
  Result := TNotModule;
end;


procedure ModuleExtInfo(Index: Long; var ExtInfo: Pointer); cdecl;
begin
  {Return module's extended info.}
  case Index of
    0: ExtInfo := @CreateIntf;
    1: ExtInfo := @CreateInst;
    2: ExtInfo := @CreateRef;
  end;
end;


{*************************************************************************}
{                         TNotModule Methods                              }
{*************************************************************************}


function TNotModule.CompressBuffer(InData: Pointer; InSize: Long;
  var OutData: Pointer; var OutSize: Long): boolean;
var
  I: Long;
begin
  Result := false;
  if (InData = nil) or (InSize = 0) then
    Exit;

  OutSize := InSize;
  { This is now big problem. Memory managers of supported languages
  behave different so meanwhile
 (until this problem is resolved ? Do you know how ?) we must use something what is the same
  already.}
  OutData := Pointer(LocalAlloc(LMEM_FIXED, OutSize));

  {Simply negate every input byte.}
  for I := 0 to InSize - 1 do
    PByteArray(OutData)[I] := not PByteArray(InData)[I];

  Result := true;
end;

function TNotModule.DecompressBuffer(InData: Pointer; InSize: Long;
  var OutData: Pointer; var OutSize: Long): boolean;
begin
  {We can call CompressBuffer because it does the same thing.}
  Result := CompressBuffer(InData, InSize, OutData, OutSize);
end;




exports
 {Export functions required by ModuleRegister.
 Names must be exact same.}
  ModuleCount,
  ModuleInfo,
  ModuleExtInfo;


begin
end.

